package com.jacquesb.planetario_a;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.TimeZone;
import java.util.Timer;
import java.util.TimerTask;

public class MainActivity extends AppCompatActivity {
    static final int INIT_VIEW = 0;
    static final int MAP_VIEW = 1;
    static final int SKY_VIEW = 2;
    PlanetarioView planetarioview = null;
    Bitmap backgroundPict;
    PGButton gotoButton;
    PGButton map;
    PGButton plus;
    PGButton minus;
    PGButton dateTimeLabel;
    PGButton increase, decrease, magnification;

    PGButton latitudeUpMover, latitudeDownMover;
    PGButton planetLabel;
    PGButton latlongLabel;
    boolean latLongInvert = false;

    SpeedFactor sf;
    Calendar utc;
    // orrery data stored in Appdata:
    // double latitude;
    // double longitude;
    // long timeInMillis is stored in AppData
    // double timeSpeedfactor is stored in AppData
    PlanetData pd;
    Planetario pt;
    ScreenCoords coords;

    String destination = "";
    boolean travelling = false;
    Toast planetMessage;

    PGButton helpUp, helpDown;
    HelpView helpview;
    boolean touch = false;
    boolean newtouch = false;
    int xt = 0;
    int yt = 0;
    int targetx, targety;
    int oldtargetx, oldtargety;

    Hashtable<String, Point> PlanetPoint = new Hashtable<String, Point>();

    //Menu menu;
    public boolean onCreateOptionsMenu(Menu menu) {
        //this.menu = menu;
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_menu, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.help:
                //show View with help instructions;
                AppData.Data.helpView = true;
                /*
                // The following causes: A/libc: Fatal signal 11 (SIGSEGV), code 1, fault addr 0xc0
                //  - without debug information...
                //
                if (! AppData.Data.helpView) {
                    AppData.Data.helpView = true;
                    //item = menu.findItem("help");
                    item.setTitle("Back");
                } else {
                    AppData.Data.helpView = false;
                    //item = menu.findItem("help");
                    item.setTitle("Help");
                }
                 */
                return true;
            case R.id.back:
                //back to planetario;
                AppData.Data.helpView = false;
                return true;
            case R.id.about:
                Toast.makeText(getBaseContext(), "Planetario v1.0\n BSD-2 clause license\n Images: CC BY-SA a.o. -> see Help",
                        Toast.LENGTH_LONG).show();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //setContentView(R.layout.activity_main);

        // get the number of pixels per inch of the screen
        AppData.Data.density = getResources().getDisplayMetrics().densityDpi;

        gotoButton = new PGButton();
        map = new PGButton();
        plus = new PGButton();
        minus = new PGButton();
        dateTimeLabel = new PGButton();
        sf = new SpeedFactor();
        increase = new PGButton();
        decrease = new PGButton();
        magnification = new PGButton();

        coords = new ScreenCoords();

        planetLabel = new PGButton();
        latlongLabel = new PGButton();

        latitudeUpMover = new PGButton();
        latitudeDownMover = new PGButton();

        helpUp = new PGButton();
        helpDown = new PGButton();

        utc = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        AppData.Data.timeInMillis = utc.getTimeInMillis();
        pt = new Planetario();
        pd = new PlanetData(pt, getResources());
        if (AppData.Data.viewState == INIT_VIEW) {
            // initialise view and orrery
            AppData.Data.viewState = MAP_VIEW;
        }

        gotoButton.tekst = "Goto ";
        gotoButton.row = PGButton.TOP_ROW;
        gotoButton.column = 0;

        map.tekst = "MAP";
        map.row = PGButton.TOP_ROW;
        map.column = 1;

        planetLabel.tekst = "Touch your location";
        planetLabel.row = PGLabel.TOP_ROW;
        planetLabel.column = 2;
        planetLabel.lastcolumn = 3;

        latlongLabel.row = PGLabel.TOP_ROW;
        latlongLabel.column = 4;
        latlongLabel.lastcolumn = 5;

        latitudeUpMover.tekst = "North";
        latitudeUpMover.row = PGLabel.TOP_ROW;
        latitudeUpMover.column = 6;

        latitudeDownMover.tekst = "Help";
        latitudeDownMover.row = PGLabel.TOP_ROW;
        latitudeDownMover.column = 7;

        minus.tekst = " << ";
        minus.row = PGButton.BOTTOM_ROW;
        minus.column = 0;

        plus.tekst = " >> ";
        plus.row = PGButton.BOTTOM_ROW;
        plus.column = 1;

        dateTimeLabel.tekst = "" + sf.speedLabel;
        dateTimeLabel.row = PGButton.BOTTOM_ROW;
        dateTimeLabel.column = 2;
        dateTimeLabel.lastcolumn = 4;

        increase.tekst = " Grow ";
        increase.row = PGButton.BOTTOM_ROW;
        increase.column = 5;

        decrease.tekst = " Shrink ";
        decrease.row = PGButton.BOTTOM_ROW;
        decrease.column = 6;

        magnification.tekst = (AppData.Data.sizefactor) + "x";
        magnification.row = PGButton.BOTTOM_ROW;
        magnification.column = 7;

        planetarioview = new PlanetarioView(this);
        setContentView(planetarioview);


        PlanetarioTimerTask myTask = new PlanetarioTimerTask();
        Timer planetarioTimer = new Timer();
        //	        public void schedule (TimerTask task, long delay, long period)
        //	        Schedule a task for repeated fixed-delay execution after a specific delay.
        //
        //	        Parameters
        //	        task  the task to schedule.
        //	        delay  amount of time in milliseconds before first execution.
        //	        period  amount of time in milliseconds between subsequent executions.

        planetarioTimer.schedule(myTask, AppData.Data.timerStepInMillis, AppData.Data.timerStepInMillis);
    }

    /**
     * converts angle from degrees to degrees-minutes-notation
     *
     * @param angle right ascension (rad)
     * @return ra in hours-notation
     */
    String angleToDegreeMinute(double angle, boolean latitude) {
        if (angle < -180) angle = angle + 360.;
        int degrees = (int) angle;
        int minutes = (int) (60 * (angle - degrees));
        if (minutes < 0) minutes = -minutes;
        //int seconds = (int)(60*(60*(angle - degrees)-minutes));
        String hemisphere = "N";
        if (latitude) {
            if (angle <= 0) hemisphere = hemisphere;
        } else {
            if (angle > 0) {
                hemisphere = "E";
            } else {
                hemisphere = "W";
            }
        }
        String degreesNS = "";
        if (Math.abs(degrees) > 10) {
            degreesNS = Math.abs(degrees) + hemisphere;
        } else {
            degreesNS = "0" + Math.abs(degrees) + hemisphere;
        }

        //return degrees + "\u00B0" + String.format("%02d", minutes) + "'";
        return degreesNS;
    }

    void printThis(Object o1) {
        System.out.println(o1);
    }

    void printThis(Object o1, Object o2) {
        System.out.println("qq " + o1 + "\t" + o2);
    }

    void printThis(Object o1, Object o2, Object o3) {
        System.out.println("qq " + o1 + "\t" + o2 + "\t" + o3);
    }

    void printThis(Object o1, Object o2, Object o3, Object o4) {
        System.out.println("qq " + o1 + "\t" + o2 + "\t" + o3 + "\t" + o4);
    }

    void printThis(Object o1, Object o2, Object o3, Object o4, Object o5) {
        System.out.println("qq " + o1 + "\t" + o2 + "\t" + o3 + "\t" + o4 + "\t" + o5);
    }

    void printThis(Object o1, Object o2, Object o3, Object o4, Object o5, Object o6) {
        System.out.println("qq " + o1 + "\t" + o2 + "\t" + o3 + "\t" + o4 + "\t" + o5 + "\t" + o6);
    }

    void printThis(Object o1, Object o2, Object o3, Object o4, Object o5, Object o6, Object o7) {
        System.out.println("qq " + o1 + "\t" + o2 + "\t" + o3 + "\t" + o4 + "\t" + o5 + "\t" + o6 + "\t" + o7);
    }

    class PlanetarioTimerTask extends TimerTask {
        public void run() {
            // mijnview.invalidate Fatal error: Android “Only the original thread that created a view hierarchy can touch its views.”

            // http://stackoverflow.com/questions/5161951/android-only-the-original-thread-that-created-a-view-hierarchy-can-touch-its-vi#5162096
            // You have to move the portion of the background task that updates the ui onto the main thread. There is a simple piece of code for this:
            // Documentation for Activity.runOnUiThread
            // Just nest this inside the method that is running in the background, then copy paste the code that implements any updates in the middle of the block.
            // Include only the smallest amount of code possible, otherwise you start to defeat the purpose of the background thread.

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    //stuff that updates ui

                    // Focus stuff
                    // Assume touch was on screen, so prepare to draw focus-cross, i.e. showFocusNextTime = true :
                    // if buttons-pressed: set showFocusNextTime to false as appropriate
                    // if zoom button pressed focus target must be set to oldtarget
                    // redraw the canvas (invalidate will call onDraw)
                    planetarioview.invalidate();
                    //sf.nextTimeInMillis();
                    sf.setNextTime();
                }
            });
        }
    }

    private class PlanetarioView extends View {

        PGButton[] planetButtons = null;
        String oldPlanet = "";
        Bitmap mapPict;
        Bitmap horizonPictDay;
        Bitmap horizonPictNight;
        boolean showFocus, showFocusNextTime;

        public PlanetarioView(Context context) {
            super(context);
            this.invalidate();

        }

        @Override
        public boolean onTouchEvent(MotionEvent event) {
            newtouch = true;
            xt = (int) event.getX();
            yt = (int) event.getY();

            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                case MotionEvent.ACTION_MOVE:
                case MotionEvent.ACTION_UP:
            }

            //AppData.Data.viewState = SKY_VIEW;

            return false;
        }

        @Override
        protected void onDraw(Canvas canvas) {
            super.onDraw(canvas);

            // ======================================================================
            // Focus stuff
            // If  showFocus = true a focus for zoom-in is drawn
            // ======================================================================

            if (showFocusNextTime) {
                showFocus = true;
            } else {
                showFocus = false;
            }
            boolean scantouch = false;
            if (newtouch) {
                touch = true;
                newtouch = false;
                showFocusNextTime = true;
                //if (targetx != oldtargetx || targety != oldtargety) scantouch = true;
                if (targetx != xt || targety != yt) scantouch = true;
                if (scantouch) {
                    //Toast.makeText(getBaseContext(), "scantouch 001", Toast.LENGTH_LONG).show();
                    Enumeration<String> planets = PlanetPoint.keys();
                    String line = "";
                    int inFocus = 0;
                    while (planets.hasMoreElements()) {
                        String planet = planets.nextElement();
                        Point pp = PlanetPoint.get(planet);
                        double distance = Math.pow(xt-pp.x,2)+Math.pow(yt-pp.y,2);
                        printThis(planet, pp.x, xt, pp.y, yt);
                        if (distance < 1.1*25*25) {
                            if (inFocus != 0) line = line +"\n";
                            line = line+planet;
                            inFocus++;
                        }
                    }
                    if (inFocus > 0) {
                        if (planetMessage!= null) {
                            planetMessage.cancel();
                        }
                        planetMessage= Toast.makeText(getBaseContext(), line, Toast.LENGTH_LONG);
                        planetMessage.show();
                    }
                }

                oldtargetx = targetx;
                oldtargety = targety;
                targetx = xt;
                targety = yt;
            } else {
                touch = false;
                newtouch = false;
            }

            if (travelling) {
                showFocusNextTime = true;
                AppData.Data.zoomfactor = AppData.Data.zoomfactor * 1.1;
                if (AppData.Data.sizefactor > 1)
                    AppData.Data.sizefactor = AppData.Data.sizefactor / 1.05;
                AppData.Data.timeSpeed = 0;
            }

            if (AppData.Data.helpView) {
                if (helpview == null) helpview = new HelpView(canvas, helpUp, helpDown);
                helpview.drawHelpText();
                if (touch) {
                    targetx = oldtargetx;
                    targety = oldtargety;
                }
                helpUp.row = PGButton.TOP_ROW;
                helpUp.column = PGButton.LAST_COLUMN - 1;
                helpUp.tekst = "Up";
                if (helpUp.pressed(xt, yt)) {
                    if (touch) {
                        helpUp.invert = true;
                        helpview.drawPreviousPage();
                        showFocusNextTime = false;
                    }
                }
                helpUp.drawButton(canvas);
                helpUp.invert = false;

                helpDown.row = PGButton.BOTTOM_ROW;
                helpDown.column = PGButton.LAST_COLUMN - 1;
                helpDown.tekst = "Down";
                if (helpDown.pressed(xt, yt)) {
                    if (touch) {
                        helpDown.invert = true;
                        helpview.drawNextPage();
                        showFocusNextTime = false;
                    }
                }
                helpDown.drawButton(canvas);
                helpDown.invert = false;

            }

            // ======================================================================
            // Canvas is redrawn a few times per second, driven by MyTimeTask myTimer
            // ======================================================================

            // MODE 1 MAP VIEW
            // step 1 draw map on canvas
            // step 2 set position on map and goto sky view

            // MODE 2 SKY VIEW
            // Part 1 draw using various draw-parameters
            // =========================================
            // step 1 draw planet horizon on canvas
            // step 2 draw planets
            // step 3 draw buttons
            // Part 2 button actions (define draw-parameters)
            // ==============================================
            // step 4 if onTouchEvent
            // step 5 set focus to focus location
            // step 6 ask each button if button is pressed
            //        if button pressed, adapt draw-parameter accordingly

            int wScreen = getWidth();
            int hScreen = getHeight();

            PlanetData.PlanetGraphics pg = pd.planetgraphicsList.get(AppData.Data.planet);

            if (AppData.Data.viewState == MAP_VIEW && !AppData.Data.helpView) {

                // MODE 1 MAP VIEW
                // step 1 draw map on canvas

                // Good place to initialise or re-initialise ScreenCoords coords:
                AppData.Data.zoomfactor = 1;
                coords.setScreen(wScreen, hScreen);
                coords.setZoomOrigin(180, 0, wScreen / 2., hScreen / 2., AppData.Data.zoomfactor);

                mapPict = pd.getPlanetPict(AppData.Data.planet, "pmap", getResources());
                int wb = mapPict.getWidth();
                int hb = mapPict.getHeight();
                int left = 0;
                int top = 0;
                int ws = 1;
                int hs = 1;
                if ((double) wb / (double) hb > (double) wScreen / (double) hScreen) {
                    ws = wScreen;
                    //hs = (int)((double)ws*(double)hb/(double)wb);
                    hs = (int) (ws / 2.);
                    top = (int) ((hScreen - hs) / 2.);
                } else {
                    hs = hScreen;
                    // ws = (int)((double)hs*(double)wb/(double)hb);
                    ws = 2 * hs;
                    left = (int) ((wScreen - ws) / 2.);
                }
                Bitmap myScaledBitmap = Bitmap.createScaledBitmap(mapPict, ws, hs, true);
                canvas.drawBitmap(myScaledBitmap, left, top, null);

                // MODE 1 MAP VIEW
                // step 2 set position on map and goto sky view

                if (AppData.Data.latitude != -1000) {
                    double xLoc = AppData.Data.longitude / 180.;
                    int xc = (int) (xLoc * (ws / 2.) + left + ws / 2.);
                    double yLoc = AppData.Data.latitude / 90.;
                    int yc = (int) (-yLoc * (hs / 2.) + top + hs / 2.);
                    Paint red = new Paint();
                    red.setColor(Color.RED);
                    canvas.drawCircle(xc, yc, 20, red);
                }
                planetLabel.drawButton(canvas);
                if (latitudeDownMover.tekst.equals("Help")) {
                    latitudeDownMover.drawButton(canvas);
                }

                if (touch) {
                    if (latitudeDownMover.pressed(xt, yt)) {
                        latitudeDownMover.invert = true;
                        latitudeDownMover.drawButton(canvas);
                        latitudeDownMover.tekst = "help-pressed";
                        Toast.makeText(getBaseContext(), "For help -> press the kebab menu (three-dots-stack) shown top right of the screen.",
                                Toast.LENGTH_LONG).show();
                        showFocusNextTime = false;
                        planetButtons = null;
                        oldPlanet = "";
                    } else {
                        double xLoc = ((double) xt - (double) left - (double) (ws / 2.)) / ((double) ws / 2.);
                        if (xLoc < -1) xLoc = -1;
                        if (xLoc > 1) xLoc = 1;
                        AppData.Data.longitude = xLoc * 180.;
                        double yLoc = ((double) (-yt) + (double) top + (double) (hs / 2.)) / ((double) hs / 2.);
                        if (yLoc < -1) yLoc = -1;
                        if (yLoc > 1) yLoc = 1;
                        AppData.Data.latitude = yLoc * 90.;
                        pd.setBestLocation(AppData.Data.planet, AppData.Data.latitude, AppData.Data.longitude);
                        AppData.Data.viewState = SKY_VIEW;
                        latitudeDownMover.invert = false;
                        latitudeDownMover.tekst = "South";
                        showFocusNextTime = false;

                    }

                }

                // MODE 1 MAP VIEW
                // remaining tasks

                map.invert = false;
                planetLabel.invert = false;
                oldPlanet = "";
                touch = false;
            }

            if (AppData.Data.viewState == SKY_VIEW && !AppData.Data.helpView) {

                // MODE 2 SKY VIEW
                // Part 1 draw using various draw-parameters
                // =========================================
                // initialise: get planet positions

                if (!AppData.Data.planet.equals(oldPlanet)) {
                    if (AppData.Data.planet.equals("Earth")) {
                        String place = "savannahsunset";
                        //String place = "antarcticascenery";
                        //if (pg.best_location != null) if (pg.best_location[0] > 15) place = "venice";
                        if (AppData.Data.latitude > 21) place = "venice";
                        if (AppData.Data.latitude < -21) place = "sydney";
                        if (Math.abs(AppData.Data.latitude) > 69) place = "antarcticascenery";
                        horizonPictDay = pd.getPlanetPict(place, "day", getResources());
                        horizonPictNight = pd.getPlanetPict(place, "night", getResources());
                    } else {
                        horizonPictDay = pd.getPlanetPict(AppData.Data.planet, "day", getResources());
                        horizonPictNight = pd.getPlanetPict(AppData.Data.planet, "night", getResources());
                    }
                    oldPlanet = AppData.Data.planet;
                }

                String[] result = null;
                if (AppData.Data.planet.equals("Olympus_Space_Station")) {
                    double[] viewdirection = {0., 0., -1.};
                    double[] horizon = {1., 0., 0.};
                    result = pt.getSky(AppData.Data.planet, viewdirection, horizon, AppData.Data.timeInMillis);
                    AppData.Data.latitude = 10;
                } else if (AppData.Data.planet.equals("Aries_Space_Station")) {
                    double[] viewdirection = {-1., 0., 0.};
                    double[] horizon = {0., 1., 0.};
                    result = pt.getSky(AppData.Data.planet, viewdirection, horizon, AppData.Data.timeInMillis);
                    AppData.Data.latitude = 10;
                } else if (AppData.Data.planet.equals("Sagittarii_Space_Station")) {
                    double[] viewdirection = {0., 1., 0.};
                    double[] horizon = {1., 0., 0.};
                    result = pt.getSky(AppData.Data.planet, viewdirection, horizon, AppData.Data.timeInMillis);
                    AppData.Data.latitude = 10;
                } else {
                    result = pt.getSky(AppData.Data.planet, AppData.Data.latitude, AppData.Data.longitude, AppData.Data.timeInMillis);
                }
                if (AppData.Data.planet.equals("Sun")) {
                    sf.setSolarDayInMillis((long) (pt.pd.planetDataTable.get("Sun").siderialDay * 24 * 3600 * 1000));
                } else {
                    sf.setSolarDayInMillis(pt.solarDayInMillis);
                }

                // MODE 2
                // step 1 draw planet horizon on canvas

                PlanetDataSource.PlanetDataStructure zon = pt.pd.planetDataTable.get("Sun");
                double sunElevation = Math.toDegrees(zon.skyLocation[1]);
                if (sunElevation < 0) {
                    backgroundPict = horizonPictNight;
                } else {
                    backgroundPict = horizonPictDay;
                }

                /**
                 *                  ___________________________________________________________
                 *  elevation = 90 |                                                           |
                 *                 |              __________________________                   |
                 *                 | yScreen=0    | (az.Origin,el.Origin)  |                   |
                 *                 |              |   x   =(xs0,ys0)       |                   |
                 *                 |              |                        |                   |
                 *                 |              |                        |                   |
                 *                 | yS = hScreen |________________________|                   |
                 *                 |         xScreen=0           xScreen=wScreen               |
                 *                 |                                                           |
                 *                 |                                                           |
                 *                 |                                                           |
                 *                 |                                                           |
                 * elevation = -90 |___________________________________________________________|
                 *       azimuth = 0                                                 azimuth = 360
                 *
                 */

                coords.setScreen(wScreen, hScreen);
                coords.getAzimuthElevation(targetx, targety);
                if (Double.isNaN(coords.azimuth)) {
                    coords.setZoomOrigin(180, 0, wScreen / 2., hScreen / 2., AppData.Data.zoomfactor);
                    coords.getAzimuthElevation(targetx, targety);
                }
                coords.setZoomOrigin(coords.azimuth, coords.elevation, targetx, targety, AppData.Data.zoomfactor);

                coords.getAzimuthElevation(0, 0);
                double elevationTop = coords.elevation;
                double azimuthLeft = coords.azimuth;

                coords.getAzimuthElevation(wScreen, hScreen);
                double elevationBottom = coords.elevation;
                double azimuthRight = coords.azimuth;

                // if azimuthLeft >= 0, azimuthRight =< 360; elevationTop =< 90 AND elevationBottom >= -90
                //   everything is OK
                // However,
                // 1) zoomOUT may lead to azimuth and/or elevation out of range
                // 2) at zoomlevel =1 (no zoom), screenratio != 1:2, so padding is needed

                double azimuthPadding = 0;
                double elevationPadding = 0;

                if (azimuthRight - azimuthLeft > 360) {
                    // leftpadding is needed
                    azimuthPadding = (azimuthRight - azimuthLeft - 360) / 2.;
                    azimuthLeft = azimuthLeft + azimuthPadding;
                    azimuthRight = azimuthRight - azimuthPadding;
                }
                if (elevationTop - elevationBottom > 180) {
                    // topPadding is needed
                    elevationPadding = (elevationTop - elevationBottom - 180) / 2.;
                    elevationTop = elevationTop - elevationPadding;
                    elevationBottom = elevationBottom + elevationPadding;
                }

                boolean inRange = true;
                double azimuthShift = 0;
                double elevationShift = 0;
                if (azimuthLeft < 0) {
                    azimuthShift = -azimuthLeft;
                    inRange = false;
                }
                if (azimuthRight > 360) {
                    azimuthShift = 360 - azimuthRight;
                    inRange = false;
                }
                if (elevationTop > 90) {
                    elevationShift = 90 - elevationTop;
                    inRange = false;
                }
                if (elevationBottom < -90) {
                    elevationShift = -90 - elevationBottom;
                    inRange = false;
                }
                // In case of zooming out the azimuth or elevation may become out of range (0 - 360 resp -90 - 90 degrees).
                // In that case the whole vie must ne shifted:
                if (!inRange) {
                    azimuthLeft = azimuthLeft + azimuthShift;
                    ;
                    azimuthRight = azimuthRight + azimuthShift;
                    elevationTop = elevationTop + elevationShift;
                    elevationBottom = elevationBottom + elevationShift;
                    // we also have to move the origin and the target
                    coords.getXscreenYscreen(coords.azimuthOrigin - azimuthShift, coords.elevationOrigin - elevationShift);
                    targetx = (int) coords.xScreen;
                    targety = (int) coords.yScreen;
                    coords.setZoomOrigin(coords.azimuthOrigin, coords.elevationOrigin, targetx, targety, AppData.Data.zoomfactor);
                }

                // draw the backgroundpicture
                int wPict = backgroundPict.getWidth();
                int hPict = backgroundPict.getHeight();

                double leftsnip = (double) wPict * azimuthLeft / 360.;
                double topsnip = (double) hPict * (90 - elevationTop) / 180;
                double wsnip = Math.max(wPict * (azimuthRight - azimuthLeft) / 360., 1);
                double hsnip = Math.max(hPict * (elevationTop - elevationBottom) / 180, 1);

                Bitmap snip = Bitmap.createBitmap(backgroundPict, (int) leftsnip, (int) topsnip, (int) wsnip, (int) hsnip);

                double leftPadding = 0;
                if (azimuthPadding > 0) {
                    double scaleFactor = (double) wScreen / (360 + 2 * azimuthPadding);
                    leftPadding = scaleFactor * azimuthPadding;
                }
                double topPadding = 0;
                if (elevationPadding > 0) {
                    double scaleFactor = (double) hScreen / (180 + 2 * elevationPadding);
                    topPadding = scaleFactor * elevationPadding;
                }
                if (leftPadding + topPadding > 0) {
                    // screen may have large white areas which are visual disturbing
                    // fill the whole screen with a background
                    Bitmap background = Bitmap.createScaledBitmap(backgroundPict, (int) wScreen, (int) hScreen, true);
                    canvas.drawBitmap(background, 0, 0, null);
                }

                Bitmap myScaledBitmap = Bitmap.createScaledBitmap(snip, (int) (wScreen - 2 * leftPadding), (int) (hScreen - 2 * topPadding), true);
                canvas.drawBitmap(myScaledBitmap, (int) leftPadding, (int) topPadding, null);
                if (AppData.Data.planet.equals("Earth")) {
                    if (AppData.Data.zoomfactor == 1) {
                        double myFontSize = 8;
                        double oneDTPdotInPixels = (0.353 / 25.4) * AppData.Data.density;
                        Paint p = new Paint();
                        p.setTextSize((float) (myFontSize * oneDTPdotInPixels));
                        if (AppData.Data.latitude > 0) {
                            p.setColor(Color.BLACK);
                            canvas.drawText("North", (float) leftPadding, (float) (hScreen / 2.), p);
                            canvas.drawText("East", (float) (leftPadding + (wScreen - 2 * leftPadding) / 4.), (float) (hScreen / 2.), p);
                            canvas.drawText("South", (float) (leftPadding + (wScreen - 2 * leftPadding) / 2.), (float) (hScreen / 2.), p);
                            canvas.drawText("West", (float) (leftPadding + 3. * (wScreen - 2 * leftPadding) / 4.), (float) (hScreen / 2.), p);
                            canvas.drawText("North", (float) (leftPadding + (wScreen - 2 * leftPadding)), (float) (hScreen / 2.), p);
                            p.setColor(Color.WHITE);
                            canvas.drawText("North", (float) (leftPadding + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("East", (float) ((leftPadding + (wScreen - 2 * leftPadding) / 4.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("South", (float) ((leftPadding + (wScreen - 2 * leftPadding) / 2.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("West", (float) ((leftPadding + 3. * (wScreen - 2 * leftPadding) / 4.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("North", (float) ((leftPadding + (wScreen - 2 * leftPadding)) + 1), (float) ((hScreen / 2.) + 1), p);
                        } else {
                            p.setColor(Color.BLACK);
                            canvas.drawText("South", (float) leftPadding, (float) (hScreen / 2.), p);
                            canvas.drawText("West", (float) (leftPadding + (wScreen - 2 * leftPadding) / 4.), (float) (hScreen / 2.), p);
                            canvas.drawText("North", (float) (leftPadding + (wScreen - 2 * leftPadding) / 2.), (float) (hScreen / 2.), p);
                            canvas.drawText("East", (float) (leftPadding + 3. * (wScreen - 2 * leftPadding) / 4.), (float) (hScreen / 2.), p);
                            canvas.drawText("West", (float) (leftPadding + (wScreen - 2 * leftPadding)), (float) (hScreen / 2.), p);
                            p.setColor(Color.WHITE);
                            canvas.drawText("South", (float) (leftPadding + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("West", (float) ((leftPadding + (wScreen - 2 * leftPadding) / 4.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("North", (float) ((leftPadding + (wScreen - 2 * leftPadding) / 2.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("East", (float) ((leftPadding + 3. * (wScreen - 2 * leftPadding) / 4.) + 1), (float) ((hScreen / 2.) + 1), p);
                            canvas.drawText("South", (float) ((leftPadding + (wScreen - 2 * leftPadding)) + 1), (float) ((hScreen / 2.) + 1), p);
                        }
                    }
                }

                if (travelling) {
                    if (AppData.Data.zoomfactor > 10) {
                        Paint p = new Paint();
                        p.setColor(Color.BLACK);
                        // zoom = 10 => alfa = 0 is transparant
                        // zoom = 110 => alfa = 255 is opaque
                        int alfa = (int) ((AppData.Data.zoomfactor - 10) * 2.55);
                        alfa = Math.min(alfa, 255);
                        p.setAlpha(alfa);
                        canvas.drawRect(0, 0, wScreen, hScreen, p);
                    }
                }

                // MODE 2
                // step 2 draw planets

                for (int iplanet = 0; iplanet < result.length; iplanet++) {
                    String planet = result[iplanet];
                    PlanetDataSource.PlanetDataStructure pds = pt.pd.planetDataTable.get(planet);
                    // PlanetDataSource.PlanetDataStructure.skyLocation[0] = longitude: CCW angle (rad) between the viewdirection and the projection of the planets vectors on the horizon
                    // PlanetDataSource.PlanetDataStructure.skyLocation[1] = latitude angle (rad) above the horizon
                    // PlanetDataSource.PlanetDataStructure.skyLocation[2] = distance (km) between the observer and the planet in the sky
                    // -> Note: with increasing longitude a point on the horizon moves to the left!
                    // azimuth is CW angle from the north
                    double azimuth = 180 - Math.toDegrees(pds.skyLocation[0]);
                    while (azimuth < 0) azimuth = azimuth + 360;
                    while (azimuth > 360) azimuth = azimuth - 360;
                    double elevation = Math.toDegrees(pds.skyLocation[1]);

                    double distance = pds.skyLocation[2];
                    // Lets assume the smartphone/tablet is at about 65 cm from your eyes, and has a screenwidth of 10 cm
                    // The circumference of a 360 degrees circle at 65 cm is 2 x PI x 65 cm = 400 cm
                    // If the full horizon (360 degrees) is shown on the screen, all objects in the sky are 400 / 10 = 40 times too small
                    // screenfactor compensates this by growing all objects in size by a factor 40.
                    AppData.Data.screenfactor = 64;

                    // in case of zoom or grow-of-planets the screenfactor should be set to 1 (here done gradually)
                    //AppData.Data.screenfactor = AppData.Data.screenfactor/(0.8*AppData.Data.zoomfactor*AppData.Data.sizefactor);
                    //if (AppData.Data.screenfactor < 1) AppData.Data.screenfactor = 1;
                    double aperture = Math.toDegrees(Math.atan(AppData.Data.sizefactor * pds.radius / distance));
                    double minRadius = 3;

                    float radius = (float) (aperture * AppData.Data.zoomfactor * (hScreen - 2 * topPadding) / 360.);
                    if (radius < minRadius) radius = (float) minRadius;

                    if (AppData.Data.latitude > 0) {
                        coords.getXscreenYscreen(azimuth, elevation);
                    } else {
                        double azimuthAlt = azimuth + 180;
                        if (azimuthAlt > 360) azimuthAlt = azimuthAlt - 360;
                        coords.getXscreenYscreen(azimuthAlt, elevation);
                    }
                    double xs = coords.xScreen;
                    double ys = coords.yScreen;

                    Point p = new Point((int)xs,(int)ys);
                    PlanetPoint.put(pds.name, p);

                    if (travelling) {
                        if (planet.equals(destination)) {
                            targetx = (int) xs;
                            targety = (int) ys;
                            if (radius > Math.max(wScreen, hScreen)) {
                                travelling = false;
                                AppData.Data.planet = destination;
                                AppData.Data.zoomfactor = 1;
                                AppData.Data.sizefactor = AppData.Data.screenfactor;
                                sf.reset();
                                PlanetData.PlanetGraphics pgsD = pd.planetgraphicsList.get(destination);
                                if (pgsD.best_location != null) {
                                    AppData.Data.latitude = pgsD.best_location[0];
                                    AppData.Data.longitude = pgsD.best_location[1];
                                }
                                if (destination.equals("Olympus_Space_Station") || destination.equals("Aries_Space_Station") || destination.equals("Sagittarii_Space_Station")) {
                                    coords.setZoomOrigin(180, 0, wScreen / 2, hScreen / 2, AppData.Data.zoomfactor);
                                    targetx = wScreen / 2;
                                    targety = hScreen / 2;
                                    showFocusNextTime = true;
                                    AppData.Data.sizefactor = 1;
                                } else {
                                    showFocusNextTime = false;
                                }
                            }
                        }
                    }

                    Paint s = new Paint();

                    PlanetData.PlanetGraphics pgs = pd.planetgraphicsList.get(planet);
                    s.setColor(pgs.color);

                    boolean draw = true;
                    if (planet.equals(AppData.Data.planet)) draw = false;
                    if (draw) {
                        if (radius < 8) {
                            canvas.drawCircle((float) xs, (float) ys, radius, s);
                        } else {
                            Rect bRect = new Rect((int) (xs - radius), (int) (ys - radius), (int) (xs + radius), (int) (ys + radius));
                            canvas.drawBitmap(pgs.planetPict, null, bRect, null);
                        }
                    }
                }

                // MODE 2
                // step 3 draw buttons

                gotoButton.drawButton(canvas);
                map.drawButton(canvas);
                planetLabel.drawButton(canvas);

                if (latLongInvert) {
                    latitudeUpMover.tekst = "West";
                    latitudeDownMover.tekst = "East";
                } else {
                    latitudeUpMover.tekst = "North";
                    latitudeDownMover.tekst = "South";
                }
                latitudeUpMover.drawButton(canvas);
                latitudeDownMover.drawButton(canvas);

                plus.drawButton(canvas);
                minus.drawButton(canvas);
                dateTimeLabel.tekst = "" + sf.speedLabel;
                dateTimeLabel.drawButton(canvas);
                if (showFocus) {
                    increase.invert = true;
                    increase.tekst = " Zoom in";
                    decrease.invert = true;
                    decrease.tekst = " Zoom out";
                    magnification.invert = true;
                } else {
                    increase.invert = false;
                    increase.tekst = " Grow ";
                    decrease.invert = false;
                    decrease.tekst = " Shrink ";
                    magnification.invert = false;
                }
                increase.drawButton(canvas);
                decrease.drawButton(canvas);
                double zoomFactor = AppData.Data.zoomfactor*AppData.Data.sizefactor / AppData.Data.screenfactor;
                double sizeFactor = AppData.Data.sizefactor;

                String zoomPostfix;
                if (zoomFactor > 1e9) {
                    zoomPostfix = (int)(zoomFactor/1e9)+"G x";
                } else if (zoomFactor > 1e6) {
                    zoomPostfix = (int)(zoomFactor/1e6)+"M x";
                } else if (zoomFactor > 1000) {
                    zoomPostfix = (int)(zoomFactor/1e3)+"k x";
                } else if (zoomFactor < 1) {
                    zoomPostfix = "1/"+(int)(1/zoomFactor)+"x";
                } else {
                    zoomPostfix = (int)zoomFactor+"x";
                }

                String sizePostfix;
                if (sizeFactor > 1e9) {
                    sizePostfix = (int)(sizeFactor/1e9)+"G x";
                } else if (sizeFactor > 1e6) {
                    sizePostfix = (int)(sizeFactor/1e6)+"M x";
                } else if (sizeFactor > 1000) {
                    sizePostfix = (int)(sizeFactor/1e3)+"k x";
                } else {
                    sizePostfix = (int)sizeFactor+"x";
                }
                
                if (showFocus) {
                    if (sizeFactor == 1) {
                        magnification.tekst = zoomPostfix;
                    } else {
                        magnification.tekst = zoomPostfix+"; G"+sizePostfix;
                    }
                } else {
                    if (zoomFactor == 1) {
                        magnification.tekst = sizePostfix;
                    } else {
                        magnification.tekst = sizePostfix+" Z"+zoomPostfix;
                    }
                }

                magnification.drawButton(canvas);

                if (!travelling) {
                    planetLabel.tekst = AppData.Data.planet;
                } else {
                    planetLabel.tekst = destination;
                }
                planetLabel.drawButton(canvas);
                String latS = angleToDegreeMinute(AppData.Data.latitude, true) + "/" + angleToDegreeMinute(AppData.Data.longitude, false);
                latlongLabel.tekst = latS;
                latlongLabel.drawButton(canvas);

                // MODE 2
                // step 4 if onTouchEvent
                // step 5 set focus to focus location
                // step 6 ask each button if button is pressed
                //        if button pressed, adapt draw-parameter accordingly

                if (gotoButton.pressed(xt, yt)) {
                    if (touch) {
                        gotoButton.invert = true;
                        gotoButton.drawButton(canvas);
                        //
                        planetButtons = new PGButton[pd.planetsList.size()];
                        int gridLength = PGButton.LAST_COLUMN / 2;
                        int bRow = 2;
                        int bColumn = 0;
                        for (int pb = 0; pb < planetButtons.length; pb++) {
                            String name = pd.planetsList.get(pb);
                            bColumn++;
                            if (bColumn > PGButton.LAST_COLUMN) {
                                bColumn = 1;
                                bRow++;
                            }
                            PGButton planetPb = new PGButton(name, bRow, bColumn - 1);
                            planetPb.lastcolumn = bColumn;
                            bColumn++;
                            planetButtons[pb] = (planetPb);
                        }
                        showFocusNextTime = false;
                    } else {
                        gotoButton.invert = false;
                    }
                }
                if (planetButtons != null) {
                    boolean pPressed = false;
                    for (int pb = 0; pb < planetButtons.length; pb++) {
                        planetButtons[pb].drawButton(canvas);
                        if (planetButtons[pb].pressed(xt, yt)) {
                            pPressed = true;
                            destination = planetButtons[pb].tekst;
                            travelling = true;
                            AppData.Data.zoomfactor = 1;
                            AppData.Data.sizefactor = AppData.Data.screenfactor;
                        }
                    }
                    if (pPressed) planetButtons = null;
                }

                if (map.pressed(xt, yt)) {
                    if (touch) {
                        map.invert = true;
                        map.drawButton(canvas);
                        AppData.Data.viewState = MAP_VIEW;
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        map.invert = false;
                    }
                }

                if (planetLabel.pressed(xt, yt)) {
                    if (touch) {
                        planetLabel.invert = true;
                        planetLabel.drawButton(canvas);
                        AppData.Data.viewState = MAP_VIEW;
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        planetLabel.invert = false;
                    }
                }

                if (latitudeUpMover.pressed(xt, yt)) {
                    if (touch) {
                        latitudeUpMover.invert = true;
                        latitudeUpMover.drawButton(canvas);
                        int iLat = (int) (AppData.Data.latitude / 10);
                        int iLong = (int) (AppData.Data.longitude / 10);
                        if (latLongInvert) {
                            iLong = iLong - 1;
                            if (iLong < -18) iLong = iLong + 36;
                        } else {
                            iLat = Math.min(9, iLat + 1);
                        }
                        AppData.Data.latitude = iLat * 10;
                        if (AppData.Data.latitude == 90) AppData.Data.latitude = 89.99999999999;
                        if (AppData.Data.latitude == -90) AppData.Data.latitude = -89.99999999999;
                        AppData.Data.longitude = iLong * 10;
                        showFocusNextTime = false;
                        planetButtons = null;
                        oldPlanet = "";
                    } else {
                        latitudeUpMover.invert = false;
                    }
                }

                if (latitudeDownMover.pressed(xt, yt)) {
                    if (touch) {
                        latitudeDownMover.invert = true;
                        latitudeDownMover.drawButton(canvas);
                        int iLat = (int) (AppData.Data.latitude / 10);
                        int iLong = (int) (AppData.Data.longitude / 10);
                        if (latLongInvert) {
                            iLong = iLong + 1;
                            if (iLong > 18) iLong = iLong - 36;
                        } else {
                            iLat = Math.max(-9, iLat - 1);
                        }
                        AppData.Data.latitude = iLat * 10;
                        if (AppData.Data.latitude == 90) AppData.Data.latitude = 89.99999999999;
                        if (AppData.Data.latitude == -90) AppData.Data.latitude = -89.99999999999;
                        AppData.Data.longitude = iLong * 10;
                        showFocusNextTime = false;
                        planetButtons = null;
                        oldPlanet = "";
                    } else {
                        latitudeDownMover.invert = false;
                    }
                }

                if (latlongLabel.pressed(xt, yt)) {
                    if (touch) {
                        latlongLabel.invert = true;
                        latlongLabel.drawButton(canvas);
                        latLongInvert = !latLongInvert;
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        latlongLabel.invert = false;
                    }
                }

                if (plus.pressed(xt, yt)) {
                    if (touch) {
                        plus.invert = true;
                        plus.drawButton(canvas);
                        sf.increaseSpeed();
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        plus.invert = false;
                    }
                }
                if (minus.pressed(xt, yt)) {
                    if (touch) {
                        minus.invert = true;
                        minus.drawButton(canvas);
                        sf.decreaseSpeed();
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        minus.invert = false;
                    }
                }
                if (dateTimeLabel.pressed(xt, yt)) {
                    if (touch) {
                        dateTimeLabel.invert = true;
                        dateTimeLabel.drawButton(canvas);
                        sf.stop();
                        showFocusNextTime = false;
                        planetButtons = null;
                    } else {
                        dateTimeLabel.invert = false;
                    }
                }

                if (increase.pressed(xt, yt)) {
                    if (touch) {
                        // set focus-target to old xt,yt:
                        targetx = oldtargetx;
                        targety = oldtargety;

                        increase.invert = true;
                        increase.drawButton(canvas);
                        planetButtons = null;
                        if (showFocus) {
                            AppData.Data.zoomfactor = AppData.Data.zoomfactor * 2;
                            AppData.Data.sizefactor = AppData.Data.sizefactor / 2;
                            if (AppData.Data.sizefactor < 1) AppData.Data.sizefactor =1;
                            increase.invert = true;
                        } else {
                            AppData.Data.sizefactor = AppData.Data.sizefactor * 2;
                            showFocusNextTime = false;
                            increase.invert = false;
                        }
                    }
                }

                if (decrease.pressed(xt, yt)) {
                    if (touch) {
                        // set focus-target to old xt,yt:
                        targetx = oldtargetx;
                        targety = oldtargety;

                        decrease.invert = true;
                        decrease.drawButton(canvas);
                        boolean decreaseSize = false;
                        boolean zoomout = false;
                        planetButtons = null;
                        if (showFocus) {
                            if (AppData.Data.zoomfactor > 1) {
                                zoomout = true;
                            } else {
                                showFocusNextTime = false;
                                decrease.invert = false;
                            }
                        } else {
                            if (AppData.Data.sizefactor > 1) {
                                decreaseSize = true;
                            } else {
                                zoomout = true;
                            }
                        }
                        if (zoomout) {
                            AppData.Data.zoomfactor = AppData.Data.zoomfactor / 2;
                            if (AppData.Data.zoomfactor < 1) AppData.Data.zoomfactor = 1;
                            if (AppData.Data.zoomfactor < AppData.Data.screenfactor) {
                                if (AppData.Data.sizefactor <  AppData.Data.screenfactor/AppData.Data.zoomfactor) {
                                    AppData.Data.sizefactor =  AppData.Data.screenfactor/AppData.Data.zoomfactor;
                                }
                            }
                        }
                        if (decreaseSize) {
                            AppData.Data.sizefactor = AppData.Data.sizefactor / 2;
                            if (AppData.Data.sizefactor < 1) AppData.Data.sizefactor = 1;
                            showFocusNextTime = false;
                            decrease.invert = false;
                        }
                    }
                }
                if (magnification.pressed(xt, yt)) {
                    if (touch) {
                        // set focus-target to old xt,yt:
                        targetx = oldtargetx;
                        targety = oldtargety;
                        if (showFocus) {
                            showFocusNextTime = false;
                        } else {
                            showFocusNextTime = true;
                        }
                        //AppData.Data.zoomfactor = 1;
                        //AppData.Data.sizefactor = 1;
                        magnification.invert = true;
                        magnification.drawButton(canvas);
                        //showFocusNextTime=false;
                        planetButtons = null;
                    } else {
                        magnification.invert = false;
                    }
                }

                Paint p = new Paint();
                if (showFocus) {
                    p.setColor(Color.WHITE);
                    canvas.drawLine(targetx - 30, targety, targetx + 30, targety, p);
                    canvas.drawLine(targetx, targety - 30, targetx, targety + 30, p);
                    p.setStyle(Paint.Style.STROKE);
                    canvas.drawCircle(targetx, targety, 25, p);
                    p.setColor(Color.BLACK);
                    canvas.drawLine(targetx - 30, targety + 1, targetx + 30, targety + 1, p);
                    canvas.drawLine(targetx + 1, targety - 30, targetx + 1, targety + 30, p);
                    p.setStyle(Paint.Style.STROKE);
                    canvas.drawCircle(targetx + 1, targety + 1, 25, p);
                }
                if (touch) {
                    p.setColor(Color.BLACK);
                    canvas.drawCircle(xt, yt, 5, p);
                }
                touch = false;

                // MODE 2
                // finalise

                if (AppData.Data.viewState == SKY_VIEW) {
                    String latS1 = angleToDegreeMinute(AppData.Data.latitude, true) + "/" + angleToDegreeMinute(AppData.Data.longitude, false);
                    latlongLabel.tekst = latS1;
                }

                // solarTime is for all planets approximately correct within 10 minutes. For earth exact calculation based on longitude:
                //if (AppData.Data.planet.equals("Earth"))
                //    solarTime = pt.SolarTimeOnEarth(AppData.Data.longitude, AppData.Data.timeInMillis);
                //canvas.drawText(new String("solarTime = "+ solarTime), (float)50, (float)600, p);
            }
        }
    }
}