package com.jacquesb.planetario_a;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.os.Build;

public class PGButton {
    static final int TOP_ROW = 1;
    static final int BOTTOM_ROW = 6;
    static final int FIRST_COLUMN = 0;
    static final int LAST_COLUMN = 8;


    String tekst;
    int row;
    int column = 1;
    int lastcolumn =0;

    double myFontSize = 8;      // 8 DTPdots
    double preferredHeightINmm = 6; // 6 mm
    float left,top,right,bottom;
    boolean invert=false;

    PGButton() {

    }
    PGButton(String tekst, int row, int column) {
        this.tekst = tekst;
        this.row = row;
        this.column = column;
    }

    void drawButton(Canvas canvas) {
        int canvasWidth = canvas.getWidth();
        int canvasHeight = canvas.getHeight();

        if (canvasWidth < 1200) if (myFontSize > 7) myFontSize = 7;
        float padding = 10;
        float strokeWidth = 5;

        // fontsize
        //   When working with font, we don't use the metric measurement system but typographic units.
        //   So when talking about font size, it's not metres and millimetres.
        //   We use but the smallest typographic unit, the point, established in the early days of printing which has survived to date.
        //   Different units of measurement were around over the years and centuries: The Didot point,
        //   a modified version based on the Fournier point, was the standard in the world of
        //   typography for a long time. At 0.375 mm, it was slightly bigger than today's
        //   PostScript or DTP point which measures 0.353 mm .
        //   So if you enter a 10 pt font size in Microsoft Word or Adobe InDesign, you
        //   define that your font will have a size of 3.53 mm.

        double oneDTPdotInPixels = (0.353 / 25.4) * AppData.Data.density;
        double preferredHeight = (preferredHeightINmm / 25.4) * AppData.Data.density;
        double maxButtonHeight = canvasHeight/BOTTOM_ROW;
        float extraPadding = 0;
        if (preferredHeight < maxButtonHeight) extraPadding = (float)((maxButtonHeight-preferredHeight)/2.);
        double maxFontSize = maxButtonHeight - 2*(padding+strokeWidth) / oneDTPdotInPixels;

        if (myFontSize > maxFontSize) myFontSize = maxFontSize;
        Paint p = new Paint();
        p.setTextSize((float) (myFontSize * oneDTPdotInPixels));

        //RectF outer = new RectF(left, top, right, bottom);
        left = column*canvasWidth/LAST_COLUMN;
        top = (float)((row - TOP_ROW)*maxButtonHeight)+extraPadding;
        bottom = (float)((1+row - TOP_ROW)*maxButtonHeight)-extraPadding;
        if (row == TOP_ROW) {
            top = 0;
            bottom = bottom-extraPadding;
        }
        if (row == BOTTOM_ROW) {
            top = top+extraPadding;
            bottom = (float)canvasHeight;
        }
        if (lastcolumn == 0) lastcolumn = column;
        right = lastcolumn*canvasWidth/LAST_COLUMN+(float)canvasWidth/(float)LAST_COLUMN;

        float tekstPadding = (bottom-top-p.getTextSize())/2f;
        float xText = left+padding+strokeWidth;
        float yText = bottom-padding-strokeWidth;

        if (! invert) {
            p.setColor(Color.LTGRAY);
        } else {
            p.setColor(Color.DKGRAY);
        }
        p.setStyle(Paint.Style.FILL);
        p.setAlpha(126);
        p.setStrokeWidth(strokeWidth);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            float rx = (bottom-top)/3;
            canvas.drawRoundRect(left + strokeWidth, top + strokeWidth, right - padding - strokeWidth, bottom - strokeWidth, rx, rx, p);
        } else {
            canvas.drawRect(left+strokeWidth, top+strokeWidth, right-padding-strokeWidth, bottom-strokeWidth, p);
        }

        if (! invert) {
            p.setColor(Color.BLACK);
        } else {
            p.setColor(Color.WHITE);
        }
        p.setStyle(Paint.Style.FILL);
        p.setAlpha(255);
        canvas.drawText(tekst, (float)left+tekstPadding, (float)bottom-tekstPadding-strokeWidth, p);

    }
    boolean pressed(int xt, int yt) {
        boolean pressed = false;
        if (xt > left) {
            if (xt < right) {
                if (yt > top) {
                    if (yt < bottom) {
                        pressed = true;
                    }
                }
            }
        }
        return pressed;
    }

}
