package com.jacquesb.planetario_a;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

public class PlanetData {

    Hashtable<String, PlanetGraphics> planetgraphicsList = new Hashtable<String, PlanetGraphics>();
    ArrayList<String> planetsList;
    Planetario pt;

    PlanetData(Planetario pt, Resources res) {
        this.pt = pt;
        planetsList = pt.pd.planetsList;
        Enumeration<String> en = pt.pd.planetDataTable.keys();
        while (en.hasMoreElements()) {
            PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
            String name = pp.name;
            PlanetGraphics pg = new PlanetGraphics(pp);
            pg.planetPict = BitmapFactory.decodeResource(res, res.getIdentifier(name.toLowerCase(), "drawable","com.jacquesb.planetario_a"));
            pg.color = Color.MAGENTA;
            //Orange #ffa500 	255 	165 	0 	38.8° 	100% 	50%
            int orange = Color.rgb(255,165,0);
            int darkblue = Color.rgb(0,76,153);
            if (name.equals("Sun")) {
                pg.color = Color.YELLOW;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }
            if (name.equals("Mercury")) {
                pg.color = orange;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }
            if (name.equals("Venus")) {
                pg.color = Color.WHITE;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }
            if (name.equals("Earth")) {
                pg.color = Color.BLUE;
                // pg.best_location
                // will be set by user
            }
            if (name.equals("Moon")) {
                pg.color = Color.LTGRAY;
                if (pg.best_location == null) {
                    // apollo 16 landing site
                    pg.best_location = new double[2];
                    pg.best_location[0] = -10.;
                    pg.best_location[1] = 16.;
                }
            }
            if (name.equals("Mars")) {
                pg.color = Color.RED;
                if (pg.best_location == null) {
                    // opportunity mars rover landing site
                    pg.best_location = new double[2];
                    pg.best_location[0] = -2;
                    pg.best_location[1] = -7;
                }
            }
            if (name.equals("Jupiter")) {
                pg.color = orange;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }
            if (name.equals("Saturn")) {
                pg.color = Color.CYAN;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }
            if (name.equals("Uranus")) {
                pg.color = darkblue;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 1;
                    pg.best_location[1] = 1;
                }
            }
            if (name.equals("Neptune")) {
                pg.color = Color.LTGRAY;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 80;
                    pg.best_location[1] = 0;
                }
            }

            if (name.equals("Pluto")) {
                pg.color = Color.GRAY;
                if (pg.best_location == null) {
                    pg.best_location = new double[2];
                    pg.best_location[0] = 1;
                    pg.best_location[1] = 1;
                }
            }
            if (name.equals("Olympus_Space_Station")) pg.color = Color.GRAY;
            if (name.equals("Aries_Space_Station")) pg.color = Color.GRAY;
            if (name.equals("Sagittarii_Space_Station")) pg.color = Color.GRAY;
            if (pg.color == Color.MAGENTA) System.out.println("qqq PlanetData.java: "+name+" has no color assigned");
            planetgraphicsList.put(name, pg);
        }
    }
    void setBestLocation(String planet, double latitude, double longitude) {
        PlanetGraphics pg = planetgraphicsList.get(planet);
        pg.best_location = new double[2];
        pg.best_location[0]=latitude;
        pg.best_location[1]=longitude;
    }
    public class PlanetGraphics {
        Bitmap planetPict;
        int color;
        double best_location[];
        PlanetDataSource.PlanetDataStructure pds;
        PlanetGraphics(PlanetDataSource.PlanetDataStructure pds) {
            this.pds = pds;
        }
    }

    /**
     * Get a backgroundpicture from the resources
     * @param planetName name of planet
     * @param mapType backgroundType: pmap, day or night
     * @param res links to resources
     * @return backgroundpicture
     */
    Bitmap getPlanetPict(String planetName, String mapType, Resources res) {
        //PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(planetName);
        Bitmap backgroundPict =  BitmapFactory.decodeResource(res, res.getIdentifier(planetName.toLowerCase()+"_"+mapType, "drawable","com.jacquesb.planetario_a"));
        return backgroundPict;
    }
}
