package com.jacquesb.planetario_a;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TimeZone;
import java.util.TreeMap;

/** <pre>
 * calculates the position of all planets in the sky
 *
 * @author jacques
 *
 */
public class Planetario {
	public static void main(String[] args) {
		System.out.println("=======================");
		System.out.println("Self-test of Planetario");
		System.out.println("=======================");
		Calendar now = Calendar.getInstance();
		System.out.println("now ="+now.get(Calendar.HOUR_OF_DAY)+":"+String.format("%02d", now.get(Calendar.MINUTE))+" "+now.get(Calendar.DAY_OF_MONTH)+"-"+(now.get(Calendar.MONTH)+1)+"-"+now.get(Calendar.YEAR));
		long nowInMillis = now.getTimeInMillis();
		Calendar utc = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
		utc.setTimeInMillis(nowInMillis);
		System.out.println("utc ="+utc.get(Calendar.HOUR_OF_DAY)+":"+String.format("%02d", utc.get(Calendar.MINUTE))+" "+utc.get(Calendar.DAY_OF_MONTH)+"-"+(utc.get(Calendar.MONTH)+1)+"-"+utc.get(Calendar.YEAR));

		Planetario pt = new Planetario();
		Enumeration<String> en = pt.pd.planetDataTable.keys();
		System.out.println("Xcoord in km in the direction of the First Point of Aries");
		System.out.println("Ycoord in km in the direction CCW of X seen from the ecliptic north");
		System.out.println("Zcoord in km in the direction of the ecliptic north");
		System.out.println("name\tXcoord\tYcoord\tzCoord\tRight Ascension\tDeclination");
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			double xyz[] = pt.getXYZ(pp.name, nowInMillis);
			double[] astronomicalCoordinates = pt.getAstronomicalCoordinates(pp.name, nowInMillis);
			System.out.println(pp.name+"\t"+xyz[0]+"\t" +xyz[1]+"\t" +xyz[2]+"\t" +pt.raToHour(astronomicalCoordinates[0])+"\t" +Math.toDegrees(astronomicalCoordinates[1]));
		}
		System.out.println("Check Right Ascension and Declination at https://theskylive.com/planets");
		System.out.println(" The fictional Aries Space Station must be near 00h00s at 0 degrees");
		System.out.println(" The fictional Sagittarii Space Station must be near 18h00s at -23 degrees near Mu Sagittarii in the constellation of Sagittarius.");
		System.out.println(" The fictional Olympus Space Station must be in the Draco constellation near Altais. See https://en.wikipedia.org/wiki/Orbital_pole: #ecliptic pole: 18h00s; 66,5 degrees");
		System.out.println();
		System.out.println("==================================================================");
		System.out.println(" Drawing of the posistion of the inner planets of the solar system");
		System.out.println(" Drawing convention: Aries to the right: X pointing to the right");
		System.out.println("                                         Y pointing up (on the screen)");
		System.out.println("===================================================================");

		int nlines = 15;
		int nchars = 90;
		ArrayList<String> asciGraph = new ArrayList<String>();
		double max = 1e9;
		en = pt.pd.planetDataTable.keys();
		TreeMap<Double, String> solarSystem = new TreeMap<Double, String>();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (! pp.name.equals("Olympus_Space_Station")) {
				double xyz[] = pt.getXYZ(pp.name, nowInMillis);
				if (xyz[0]*xyz[0]+xyz[1]*xyz[1]<0.25*max*max) {
					double y = 0.5*xyz[1]/max;
					double line = (int)(nlines*(0.5-y));
					double x = 0.5*xyz[0]/max;
					double chr = (int)(nchars*(0.5+x));
					String sline="";
					for (int j=0;j<chr;j++) {
						sline = sline+" ";
					}
					sline = sline+pp.name;
					solarSystem.put(-xyz[1], sline);
				}
			}
		}
		// Display elements
		Set<Entry<Double,String>> set = solarSystem.entrySet();
		Iterator<Entry<Double, String>> jj = set.iterator();
		while(jj.hasNext()) {
			Map.Entry<Double,String> me = (Map.Entry<Double,String>)jj.next();
			System.out.println(me.getValue());
		}

		System.out.println("Check at https://theskylive.com/3dsolarsystem");
		System.out.println("===================================================================");
		double[] earth = pt.getXYZ("Earth", nowInMillis);
		double[] mars = pt.getXYZ("Mars", nowInMillis);
		double[] raDecl = pt.getRAandDeclination(mars, earth);
		System.out.println("Mars: "+(pt.raToHour(raDecl[0]))+", "+Math.toDegrees(raDecl[1])+" degrees");
		System.out.println("Check result at (e.g.) https://theskylive.com/mars-info");
		double[] sun = pt.getXYZ("Sun", nowInMillis);
		raDecl = pt.getRAandDeclination(sun, earth);
		System.out.println("Sun: "+(pt.raToHour(raDecl[0]))+", "+Math.toDegrees(raDecl[1])+" degrees");
		sun = pt.getXYZ("Sun", nowInMillis+(long)(1*24*3600*1000));
		earth = pt.getXYZ("Earth", nowInMillis+(long)(1*24*3600*1000));
		raDecl = pt.getRAandDeclination(sun, earth);
		System.out.println("Sun tomorrow: "+(pt.raToHour(raDecl[0]))+", "+Math.toDegrees(raDecl[1])+" degrees");
		System.out.println("Check result at (e.g.) https://theskylive.com/sun-info#ephemeris");
		System.out.println("=========================================================================================================================");

		System.out.println("");		
		double[] moon = pt.getXYZ("Moon", nowInMillis);
		earth = pt.getXYZ("Earth", nowInMillis);
		raDecl = pt.getRAandDeclination(moon, earth);
		System.out.println("Moon now: "+(pt.raToHour(raDecl[0]))+", "+Math.toDegrees(raDecl[1])+" degrees");
		moon = pt.getXYZ("Moon", nowInMillis+(long)(1*24*3600*1000));
		earth = pt.getXYZ("Earth", nowInMillis+(long)(1*24*3600*1000));
		raDecl = pt.getRAandDeclination(moon, earth);
		System.out.println("Moon tomorrow: "+(pt.raToHour(raDecl[0]))+", "+Math.toDegrees(raDecl[1])+" degrees");
		System.out.println("Check result at (e.g.) https://theskylive.com/moon-info#ephemeris");
		System.out.println("=========================================================================================================================");

		pt.diag = false;
		double latitude = 0;
		double longitude = 0;
		String planet = "Earth";
		double solarTime;
		for (int j = 0; j<4;j++) {
			long timeInMillis=0;
			if (j== 0) {
				System.out.println("The sky in Koedijk (sketchy presentation)");
				timeInMillis=nowInMillis;
				latitude = 52.6741;
				longitude = 4.753;
				planet = "Earth";
			} else if (j==1) {
				System.out.println("The sky in Koedijk next midnight)");
				solarTime = pt.solarTime;
				long millisecondsToNextMidnight = pt.millisecondsToNextMidnight(solarTime, pt.pd.planetDataTable.get(planet));
				timeInMillis=nowInMillis+millisecondsToNextMidnight;
				latitude = 52.6741;
				longitude = 4.753;
				planet = "Earth";
			} else if (j==2) {
				System.out.println("The sky in Koedijk in 90 days (sketchy presentation)");
				timeInMillis=nowInMillis+(90L*24L*3600L*1000L);
				latitude = 52.6741;
				longitude = 4.753;
				planet = "Earth";
			} else {
				System.out.println("=========================================================================================================================");
				System.out.println("The sky in Sidney (sketchy presentation)");
				timeInMillis=nowInMillis;
				latitude = -33.7;
				longitude = 151;
				planet = "Earth";
			}
			// PlanetDataSource.PlanetDataStructure.skyLocation[0] = azimuthAlternative: CCW angle (rad) between the viewdirection and the projection of the planets vectors on the horizon
			// PlanetDataSource.PlanetDataStructure.skyLocation[1] = elevation angle (rad) above the horizon
			// PlanetDataSource.PlanetDataStructure.skyLocation[2] = distance (km) between the observer and the planet in the sky
			// -> Note: with increasing azimuthAlternative a point on the horizon moves to the left!

			String[] result = pt.getSky(planet, latitude,longitude, timeInMillis);
			solarTime = pt.solarTime;
			TreeMap<Double, String> sky = new TreeMap<Double, String>();
			double zero = 0;
			sky.put(zero,"East--------------------------South-------------------------West---------------------------North-------------------------East");
			//           .---------0---------0---------0                              .---------0---------0---------0
			//                                         .---------0---------0---------0                              .---------0---------0---------0

			en = pt.pd.planetDataTable.keys();
			while (en.hasMoreElements()) {
				PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
				if (! pp.name.equals(planet)) {
					//viedirection is south
					//azimuthAlternative = 0 is south:
					//azimuthAlternative PI is east
					//azimuthAlternative -PI is west
					int iangle = (int)Math.toDegrees(-pp.skyLocation[0])+ 90;
					while (iangle < 0) iangle = iangle+360;
					while (iangle > 360) iangle = iangle-360;
					iangle = iangle/3;
					String line = "";
					for (int i = 0; i < iangle; i++) line = line +" ";
					line = line + pp.name;
					sky.put(-pp.skyLocation[1], line);
				}
			}
			Set<Entry<Double,String>> set1 = sky.entrySet();
			Iterator<Entry<Double, String>> i = set1.iterator();
			// Display elements
			while(i.hasNext()) {
				Map.Entry<Double,String> me = (Map.Entry<Double,String>)i.next();
				//System.out.print((-me.getKey()) + ": ");
				System.out.println(me.getValue());
			}
			if (j== 0) System.out.println("Check at https://stellarium-web.org/   Set location to Alkmaar, The Netherlands");
			if (j>2) System.out.println("Check at https://stellarium-web.org/   Set location to Sydney, Australia");
			
			sky = new TreeMap<Double, String>();
			en = pt.pd.planetDataTable.keys();
			while (en.hasMoreElements()) {
				PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
				if (! pp.name.equals(planet)) {
					//viedirection is south
					//azimuthAlternative = 0 is south:
					//azimuthAlternative PI/2 is east
					//azimuthAlternative -PI/2 is west
					
					// The azimuth is the angle formed between a reference direction (in this example north) and 
					// a line from the observer to a point of interest projected on the same plane as the reference 
					// direction orthogonal to the zenith.
					// In modern astronomy azimuth is nearly always measured from the north.
					
					//azimuthAlternative PI (or -PI) is north -> azimuth = 0 
					//azimuthAlternative PI/2 is east         -> azimuth = 90 
					//azimuthAlternative = 0 is south         -> azimuth = 180
					//azimuthAlternative -PI/2 is west        -> azimuth = 270 (0r -90)
					
					
					double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
					while (azimuth > 360) azimuth = azimuth - 360;
					while (azimuth < 0) azimuth = azimuth + 360;

					
					int iangle = (int)Math.toDegrees(-pp.skyLocation[0])+ 90;
					while (iangle < 0) iangle = iangle+360;
					while (iangle > 360) iangle = iangle-360;
					sky.put(azimuth,pp.name+"\t"+azimuth+"\t"+Math.toDegrees(pp.skyLocation[1]));
				}
			}
			Set<Entry<Double,String>> set2 = sky.entrySet();
			Iterator<Entry<Double, String>> ii = set2.iterator();
			// Display elements
			System.out.println("Planet\tAzimuth\tElevation ");
			while(ii.hasNext()) {
				Map.Entry<Double,String> me2 = (Map.Entry<Double,String>)ii.next();
				//System.out.print((-me.getKey()) + ": ");
				System.out.println(me2.getValue());
			}

			System.out.println("Local solar time is "+pt.solarTimeString);
			pt.SolarTimeOnEarth(longitude, timeInMillis);
			
			System.out.println("=========================================================================================================================");
		}


		String viewpoint = "Olympus_Space_Station";
		System.out.println();
		System.out.println();

		for (int i =0; i < 3; i++) {
			viewpoint = "Olympus_Space_Station";
			double[] viewdirection = {0.,0.,-1};
			double[] horizon = {1.,0., 0.};
			String[] result;
			if (i == 0 ) {
				System.out.println("==================================================================");
				System.out.println(" Drawing of the posistion of the planets seen from "+viewpoint);
				System.out.println(" Drawing convention: Aries to the right: X pointing to the right");
				System.out.println("                                         Y pointing up (on the screen)");
				System.out.println("===================================================================");
				result = pt.getSky(viewpoint,viewdirection, horizon,nowInMillis);
			} else if (i == 1) {
				viewpoint = "Aries_Space_Station";
				System.out.println("==================================================================");
				System.out.println(" Drawing of the posistion of the planets seen from "+viewpoint);
				System.out.println("                        Y pointing up (on the screen) is away from Aries");
				System.out.println("===================================================================");
				result = pt.getSky(viewpoint,new double[] {-1,0,0}, new double[] {0,1,0},nowInMillis);
			} else if (i == 2) {
				viewpoint = "Sagittarii_Space_Station";
				System.out.println("==================================================================");
				System.out.println(" Drawing of the posistion of the planets seen from "+viewpoint);
				System.out.println("                        Y pointing up (on the screen) is away from Piscium");
				System.out.println("===================================================================");
				result = pt.getSky(viewpoint,new double[] {0,1,0}, new double[] {1,0,0},nowInMillis);
			} else {
				//do nothing
			}
			nlines = 18;
			nchars = 120;
			String[] lines = new String[nlines];
			for (int j = 0;j < nlines; j++) {
				lines[j] = "";
				for (int k = 0; k < nchars+2; k++ ) {
					lines[j] = lines[j]+" ";
				}
			}

			solarSystem = new TreeMap<Double, String>();
			en = pt.pd.planetDataTable.keys();
			while (en.hasMoreElements()) {
				PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
				if (! pp.name.equals(viewpoint)) {
					//viedirection is to the sun, seen from positive X (aries) or positive Y (ecliptic north)
					//pp.skyLocation[0] = 'azimuthAlternative' = 0 is to the sun:
					//pp.skyLocation[0] = 'azimuthAlternative' = PI is to the left of the sun
					//pp.skyLocation[0] = 'azimuthAlternative' = -PI is to the right of the sun
					//==> pp.skyLocation[0] = -X coordinate
					//pp.skyLocation[1] = Y-coordinate

					double x = Math.toDegrees(-pp.skyLocation[0]); 
					double y = Math.toDegrees(pp.skyLocation[1]); 

					// X: <-180,180] = 120 chars => 3 degrees per char: 180-177 = char 0; 177 - 174 = char 1; etc.
					// Y: <-90, 90] => 18 lines => 10 degrees per line: 90-80 = line 0; 80-70 = line 1; etc.

					double xstep = 360/nchars;
					int nx = (int)((x + 180)/xstep);
					while (nx < 0) nx = nx + nchars;
					while (nx >= 120) nx = nx - nchars;

					double ystep = 180/nlines;
					int ny = (int)((90-y)/ystep);
					while (ny < 0) ny = ny + nlines;
					while (ny >= nlines) ny = ny - nlines;

					// Write pp.name to lines(ny) if lines(ny).subString(nx,1) is empty (or better, is not an upperCase:
					while (Character.isUpperCase(lines[ny].charAt(nx))) nx = nx+1;
					int len = 3;
					if (Character.isUpperCase(lines[ny].charAt(nx+2))) len=2;
					if (Character.isUpperCase(lines[ny].charAt(nx+1))) len=1;
					lines[ny]=lines[ny].substring(0,nx)+pp.name.substring(0,len)+lines[ny].substring(nx+len);
				}
			}
			// Display elements
			for (int j = 0;j < nlines; j++) {
				System.out.println(lines[j]);
			}
			System.out.println("Check at https://theskylive.com/3dsolarsystem");
		}

		System.out.println("=========================================================================================================================");

		System.out.println("Earth seenfrom the apollo 16 landing site on the moon");
		System.out.println(" since earth is not on the moon surface images of the apollo misisons, it is estimated that earth should have an elevation of more than about 60 degrees");
		
		System.out.println("date       azimuth            elevation           distance");
		for (int i =0; i < 24; i++) {
			long timeInMillis = nowInMillis+(long)(i*30.5*24*3600*1000);
			Calendar thisTime = Calendar.getInstance(TimeZone.getDefault());
			thisTime.setTimeInMillis(timeInMillis);
			String[] result = pt.getSky("Moon", -10, 16, timeInMillis);
			en = pt.pd.planetDataTable.keys();
			while (en.hasMoreElements()) {
				PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
				if (pp.name.equals("Earth")) {
					double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
					while (azimuth > 360) azimuth = azimuth - 360;
					while (azimuth < 0) azimuth = azimuth + 360;
					String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH);		
					System.out.println(date+" "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				}
			}
		}


		System.out.println("=========================================================================================================================");

		System.out.println("Moon seen from Koedijk");
		System.out.println("date                 azimuth            elevation           distance");
		
		// Moon  seen from koedijk https://stellarium-web.org/
		//                     azimuth elevation
		// 11:47:06 2023-01-03 25o48' -11o40'
		// 11:47:06 2023-01-04 14o19' -11o30'
		// 11:47:06 2023-01-05 2o59' -11o07'
		// 11:47:06 2023-01-12 285o27' -3o55'
		// 11:47:06 2024-01-05 242o27' 04o40'
		// 11:47:06 2026-01-05 318o22' -09o31'
		
		long timeInMillis = 1672743920163L-18L*60L*1000L;
		Calendar thisTime = Calendar.getInstance(TimeZone.getDefault());
		thisTime.setTimeInMillis(timeInMillis);
		String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);
		
		latitude = 52.6741;
		longitude = 4.753;
		String[] result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   25o48'            -11o40'");
			}
		}
		
		timeInMillis = 1672743920163L-18L*60L*1000L+24L*3600L*1000L;
		thisTime.setTimeInMillis(timeInMillis);
		date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);
		
		latitude = 52.6741;
		longitude = 4.753;
		result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   14o19'            -11o30");
			}
		}

		timeInMillis = 1672743920163L-18L*60L*1000L+2L*24L*3600L*1000L;
		thisTime.setTimeInMillis(timeInMillis);
		date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);
		
		latitude = 52.6741;
		longitude = 4.753;
		result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   2o59'             -11o07");
			}
		}
		
		timeInMillis = 1672743920163L-18L*60L*1000L+9L*24L*3600L*1000L;
		thisTime.setTimeInMillis(timeInMillis);
		date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);

		latitude = 52.6741;
		longitude = 4.753;
		result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   285o27'           -3o55' ");
			}
		}
		
		// 11:47:06 2024-01-05 242o27' 04o40'
		timeInMillis = 1672743920163L-18L*60L*1000L+367L*24L*3600L*1000L;
		thisTime.setTimeInMillis(timeInMillis);
		date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);

		latitude = 52.6741;
		longitude = 4.753;
		result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   242o27'           04o40' ");
			}
		}

		// 11:47:06 2026-01-05 318o22' -09o31'
		timeInMillis = 1672743920163L-18L*60L*1000L+1098L*24L*3600L*1000L;
		thisTime.setTimeInMillis(timeInMillis);
		date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
		//System.out.println(timeInMillis+" "+date);
		System.out.println(date);

		latitude = 52.6741;
		longitude = 4.753;
		result = pt.getSky("Earth", latitude,longitude, timeInMillis);
		en = pt.pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pt.pd.planetDataTable.get(en.nextElement());
			if (pp.name.equals("Moon")) {
				double azimuth = Math.toDegrees(-pp.skyLocation[0]) + 180;
				while (azimuth > 360) azimuth = azimuth - 360;
				while (azimuth < 0) azimuth = azimuth + 360;
				//String date = ""+thisTime.get(Calendar.YEAR)+"-"+(thisTime.get(Calendar.MONTH)+1)+"-"+thisTime.get(Calendar.DAY_OF_MONTH)+" "+thisTime.get(Calendar.HOUR_OF_DAY)+":"+thisTime.get(Calendar.MINUTE);		
				System.out.println("planetario           "+azimuth+" "+Math.toDegrees(pp.skyLocation[1])+" "+pp.skyLocation[2]);
				System.out.println("stellarium-web.org   318o22'           -09o31");
			}
		}

		System.out.println("=========================================================================================================================");

		System.out.println("End self-test of Planeterio");

	}
	boolean diag = false;
	/**  <pre>
	 * calculates the number of milliseconds to the next midnight on myPlanet
	 * @param solarTime  number of local solar hours (local observer planet-hours != earth hours) since last midnight
	 * @param myPlanet   data of the observer's planet 
	 * @return earth-milliseconds to next midnight
	 */
	public long millisecondsToNextMidnight(double solarTime, PlanetDataSource.PlanetDataStructure myPlanet) {
		// a siderial hour is for non-retrogade planets slightly shorter than a solar hour
		// a planet has 24 planet-hours per solar day,  1 planet-hour != 1 earth hour
		double siderialDay = myPlanet.siderialDay; // in solarEarthDays - negative for retrogade planets 
		double solarYear = myPlanet.orbitalPeriod*pd.daysPerYear; // in solarEarthDays
		double siderialDaysPerYear = solarYear/siderialDay;
		double solarDaysPerYear = siderialDaysPerYear-1;
		//double solarEarthDaysPerSolarDay = solarYear/solarDaysPerYear;
		//double solarEarthHoursPerSolarHour = solarYear/solarDaysPerYear;
		double millisecondsPerSolarHour = solarYear*3600*1000/solarDaysPerYear;
		long millisecondsToNextMidnight = (long)((24.-solarTime)*millisecondsPerSolarHour);
		return millisecondsToNextMidnight;
	}
	/**  <pre>
	 * calculates the number of milliseconds to the previous midnight on myPlanet
	 * @param solarTime  number of local solar hours (local observer planet-hours != earth hours) since last midnight
	 * @param myPlanet   data of the observer's planet 
	 * @return earth-milliseconds to previous midnight
	 */
	public long millisecondsToPreviousMidnight(double solarTime, PlanetDataSource.PlanetDataStructure myPlanet) {
		// a siderial hour is for non-retrogade planets slightly shorter than a solar hour
		// a planet has 24 planet-hours per solar day,  1 planet-hour != 1 earth hour
		double siderialDay = myPlanet.siderialDay; // in solarEarthDays - negative for retrogade planets 
		double solarYear = myPlanet.orbitalPeriod*pd.daysPerYear; // in solarEarthDays
		double siderialDaysPerYear = solarYear/siderialDay;
		double solarDaysPerYear = siderialDaysPerYear-1;
		//double solarEarthDaysPerSolarDay = solarYear/solarDaysPerYear;
		//double solarEarthHoursPerSolarHour = solarYear/solarDaysPerYear;
		double millisecondsPerSolarHour = solarYear*3600*1000/solarDaysPerYear;
		long millisecondsToPreviousMidnight = (long)(solarTime*millisecondsPerSolarHour);
		return millisecondsToPreviousMidnight;
	}
	/**  <pre>
	 * get the angle theta between two vectors a and b by using the dotproduct a.b = |a| |b| cos(theta)
	 * @param a 3D vector
	 * @param b 3D vector
	 * @return angle between a and b: [0,PI> rads
	 */
	private double angle(double[] a, double[] b) {
		double dotproduct = a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
		double lengtha = Math.sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
		double lengthb = Math.sqrt(b[0]*b[0]+b[1]*b[1]+b[2]*b[2]);
		double cos_angle = dotproduct/(lengtha*lengthb);
		double angle = Math.acos(cos_angle);
		return angle;
	}

	PlanetDataSource pd;
	/** rotates a vector coordinates relating to the ecliptic plane such that the coordinates relate to the earth equator plane */
	private Rotator incline;
	public Planetario() {
		pd = new PlanetDataSource();
		incline = new Rotator(new double[] {1,0,0}, Math.toRadians(23.4392811));
	}
	double solarTime;
	String solarTimeString;
	long solarDayInMillis;
	/**  <pre>
	 * Find the location of the planets for a person at location XYZ viewing in direction XYZ and time.
	 * results are set in the instance of the class PlanetDataSource in the key double[3] skyLocation
	 * PlanetDataSource.PlanetDataStructure.skyLocation[0] = azimuthAlternative: CCW angle (rad) between the viewdirection and the projection of the planets vectors on the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[1] = elevation angle (rad) above the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[2] = distance (km) between the observer and the planet in the sky
	 * -> Note: with increasing azimuthAlternative a point on the horizon moves to the left!
	 * 
	 * @param location observer at location XYZ
	 * @param viewDirection viewing in direction XYZ
	 * @param horizon positive X-axis in XYZ (horizon and viewdirection must be perpendicular) 
	 * @param timeInMillis	timeInMillis (UTC as produced by java.Calendar)
	 * @param myPlanet obervers planet or null -> null gives dummy solartime
	 * @return String[] with solarTime and a list of planet names in descending distance order 
	 *
	 */
	private String[] getSky(double[] location, double[] viewDirection, double[] horizon, long timeInMillis,  PlanetDataSource.PlanetDataStructure myPlanet) {

		//set up a rotator rp that rotates viewdirection vector to {0,0,-1}
		// that way all far away objects in the view direction can be viewed in the x,y plane and are near to {x,y} = {0,0}
		// note that the horizon vector after rotation with rp must now be in the form {x, y, 0}, since it is perpendicular to viewdirection   

		//set up a rotator rp2 such that the rp-rotated horizon vector rotates to {1,0,0}
		//  note that the rotated viewdirection {0,0,-1} does not change by this rotation.

		// For all vectors, after rotation rp and rp2: 
		//    the y-component giver the elevation above the horizon --> atan2(y, sqrt(x**2+z**2)
		//    the -x over -z ratio determines the azimuthAlternative --> atan2(-x, -z) 
		//    -> Note: with increasing azimuthAlternative a point on the horizon moves to the left!


		Rotator rp = new Rotator(viewDirection, new double[] {0,0,-1});

		double[] west = rp.rotate(horizon);
		//Rotator rp2 = new Rotator(horizon,aries);
		Rotator rp2 = new Rotator(west,new double[] {1,0,0});

		// dus:
		//   rp2(rp(viewDirection)) --> rp2{0,0,-1} --> ??
		//   rp2(rp(horizon))--> {1,0,0}


		if (diag) {
			System.out.println("\nviewdirection = {"+viewDirection[0]+","+viewDirection[1]+","+viewDirection[2]+"}");
			System.out.println("horizon = {"+horizon[0]+","+horizon[1]+","+horizon[2]+"}");

			double[] vd = rp.rotate(viewDirection);

			System.out.println("rp viewdirection = {"+vd[0]+","+vd[1]+","+vd[2]+"} must be {0, 0, -1}");
			System.out.println("rp horizon = {"+west[0]+","+west[1]+","+west[2]+"} must be {x, y, 0}");

			vd = rp2.rotate(vd); 
			System.out.println("rp2 rp viewdirection = {"+vd[0]+","+vd[1]+","+vd[2]+"} must be {0, 0, -1}");
			double h2[] = rp2.rotate(west);
			System.out.println("rp2 rp horizon = {"+h2[0]+","+h2[1]+","+h2[2]+"} must be {1, 0, 0}\n");

		}
		Enumeration<String> en = pd.planetDataTable.keys();
		ArrayList<String> tobeSortedName = new ArrayList<String>();
		ArrayList<Double> tobeSortedValue = new ArrayList<Double>();
		solarTime = -1.;
		while (en.hasMoreElements()) {
			PlanetDataSource.PlanetDataStructure pp = pd.planetDataTable.get(en.nextElement());
			double xyz[] = getXYZ(pp.name, timeInMillis);
			double[] xyzLoc = new double[3];
			xyzLoc[0] = xyz[0]-location[0];
			xyzLoc[1] = xyz[1]-location[1];
			xyzLoc[2] = xyz[2]-location[2];
			if (myPlanet != null) {
				if (pp.name.equals("Sun")) {
					solarTime = solarTime(xyzLoc, viewDirection, myPlanet);
				}
			}

			double[] xyzSky = rp.rotate(xyzLoc);
			if (pp.name.equals("Sun")&&diag) {
				System.out.println("\nSun = {"+xyzLoc[0]+","+xyzLoc[1]+","+xyzLoc[2]+"}");
				System.out.println("rp Sun = {"+xyzSky[0]+","+xyzSky[1]+","+xyzSky[2]+"}");
			}
			xyzSky = rp2.rotate(xyzSky);
			if (pp.name.equals("Sun")&&diag) {
				System.out.println("rp2 rp Sun = {"+xyzSky[0]+","+xyzSky[1]+","+xyzSky[2]+"}");
			}

			double azimuthAlternative = Math.atan2(-xyzSky[0], -xyzSky[2]);  
			double elevation = Math.atan2(xyzSky[1], Math.sqrt(xyzSky[0]*xyzSky[0]+xyzSky[2]*xyzSky[2]));
			if (pp.name.equals("Sun")&&diag) {
				System.out.println("\nSun elevation = "+Math.toDegrees(elevation));
				System.out.println("Sun azimuthAlternative = "+Math.toDegrees(azimuthAlternative)+" degrees to the left(CCW!) from the center viewpoint (often south)");
				double azimuth = 180-Math.toDegrees(azimuthAlternative);
				while (azimuth < -180) azimuth = azimuth+360; 
				while (azimuth > 180) azimuth = azimuth-360; 
				System.out.println("Sun azimuth = "+azimuth+" degrees (azimuth is CW angle from the north)\n");
			}

			pp.skyLocation[0] = azimuthAlternative;
			pp.skyLocation[1] = elevation;
			pp.skyLocation[2] = Math.sqrt(xyzSky[0]*xyzSky[0]+xyzSky[1]*xyzSky[1]+xyzSky[2]*xyzSky[2]);

			//System.out.println("longit "+pp.name+" "+Math.toDegrees(pangle));
			tobeSortedName.add(pp.name);
			tobeSortedValue.add(pp.skyLocation[2]);
		}
		solarTimeString = this.solarTimeToString(solarTime);

		// a siderial hour is for non-retrogade planets slightly shorter than a solar hour
		// a planet has 24 planet-hours per solar day,  1 planet-hour != 1 earth hour
		double siderialDay = myPlanet.siderialDay; // in solarEarthDays - negative for retrogade planets 
		double solarYear = myPlanet.orbitalPeriod*pd.daysPerYear; // in solarEarthDays
		double siderialDaysPerYear = solarYear/siderialDay;
		double solarDaysPerYear = siderialDaysPerYear-1;
		//double solarEarthDaysPerSolarDay = solarYear/solarDaysPerYear;
		//double solarEarthHoursPerSolarHour = solarYear/solarDaysPerYear;
		double millisecondsPerSolarHour = solarYear*3600*1000/solarDaysPerYear;
		long millisecondsToNextMidnight = (long)((24.-solarTime)*millisecondsPerSolarHour);
		solarDayInMillis = (long)(24.*millisecondsPerSolarHour);

		// create a list of planet names sorted according to distance to the observer
		String[] result = new String[pd.planetDataTable.size()];
		double oldMinDistance = -1;
		String pld = "";
		for (int i=0; i < result.length; i++) {
			double newMinDistance = -1;
			int jNewMin = 0;
			for (int j=0; j < tobeSortedValue.size(); j++) {
				if (newMinDistance == -1) {
					newMinDistance = tobeSortedValue.get(j);
					pld = tobeSortedName.get(j);
					jNewMin = j;
				}
				if (tobeSortedValue.get(j) >= oldMinDistance) {
					if (tobeSortedValue.get(j) < newMinDistance) {
						newMinDistance = tobeSortedValue.get(j);
						pld = tobeSortedName.get(j);
						jNewMin = j;
					}
				}
			}
			oldMinDistance = newMinDistance;
			tobeSortedValue.remove(jNewMin);
			tobeSortedName.remove(jNewMin);
			result[result.length-1-i] = pld;
		}
		return result;
	}
	/**  <pre>
	 * Get the astromomical coordinates of a planet at timeInMillis 
	 * @param planet name of the planet
	 * @param timeInMillis the number of milliseconds that have elapsed since the Unix epoch, excluding leap seconds. The Unix epoch is 00:00:00 UTC on 1 January 1970.
	 * @return {rightAscension, declination} both in rad
	 */
	public double[] getAstronomicalCoordinates(String planet, long timeInMillis) {
		PlanetDataSource.PlanetDataStructure myPlanet = pd.planetDataTable.get(planet);
		double[] myPlanetXYZ = getXYZ(planet, timeInMillis);
		PlanetDataSource.PlanetDataStructure earth = pd.planetDataTable.get("Earth");
		double[] earthXYZ = getXYZ("Earth", timeInMillis);
		double[] xyzLoc = new double[3];
		for (int i = 0; i<3; i++) xyzLoc[i] = myPlanetXYZ[i]-earthXYZ[i];

		//System.out.println(planet+"\t"+myPlanetXYZ[0]+"\t"+myPlanetXYZ[1]+"\t"+myPlanetXYZ[2]);
		//System.out.println("Earth"+"\t"+earthXYZ[0]+"\t"+earthXYZ[1]+"\t"+earthXYZ[2]);

		Rotator asco = new Rotator(earth.rotationAxis, new double[] {0,0,1});  
		myPlanetXYZ = asco.rotate(xyzLoc);

		double rightAscension=Math.atan2(myPlanetXYZ[1], myPlanetXYZ[0]);
		double declination=Math.atan2(myPlanetXYZ[2], Math.sqrt(myPlanetXYZ[0]*myPlanetXYZ[0]+myPlanetXYZ[1]*myPlanetXYZ[1]));
		double[] astronomicalCoordinates = new double[2];
		astronomicalCoordinates[0]=rightAscension;
		astronomicalCoordinates[1]=declination;
		return astronomicalCoordinates;
	}
	/**  <pre>
	 * Find the location of the planets for a person on a planet viewing in direction XYZ and time.
	 * results are set in the instance of the class PlanetDataSource in the key double[3] skyLocation
	 * PlanetDataSource.PlanetDataStructure.skyLocation[0] = azimuthAlternative: CCW angle (rad) between the viewdirection and the projection of the planets vectors on the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[1] = elevation angle (rad) above the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[2] = distance (km) between the observer and the planet in the sky
	 * -> Note: with increasing azimuthAlternative a point on the horizon moves to the left!
	 * @param planet	name of the planet
	 * @param viewDirection viewing in direction XYZ
	 * @param horizon positive X-axis in XYZ
	 * @param timeInMillis	timeInMillis (UTC as produced by java.Calendar)
	 * @return String[] with solarTime and a list of planet names in descending distance order 
	 *
	 */
	public String[] getSky(String planet,   double[] viewDirection, double[] horizon, long timeInMillis) {
		PlanetDataSource.PlanetDataStructure myPlanet = pd.planetDataTable.get(planet);
		double[] myPlanetXYZ = getXYZ(planet, timeInMillis);
		String[] result = getSky(myPlanetXYZ, viewDirection, horizon, timeInMillis, myPlanet);
		return result;
	}
	/**  <pre>
	 * Find the location of the planets in the sky of a person on a planet at given latitude and longitude and time, where the person is looking south
	 * results are set in the instance of the class PlanetDataSource in the key double[3] skyLocation
	 * PlanetDataSource.PlanetDataStructure.skyLocation[0] = azimuthAlternative: CCW angle (rad) between the viewdirection and the projection of the planets vectors on the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[1] = elevation angle (rad) above the horizon
	 * PlanetDataSource.PlanetDataStructure.skyLocation[2] = distance (km) between the observer and the planet in the sky
	 * -> Note: with increasing azimuthAlternative a point on the horizon moves to the left!
	 * @param planet	name of the planet
	 * @param latitude  angle ranging from 0\B0 at the Equator to 90\B0 at the North Pole and -90\B0 at the South Pole
	 * @param longitude	angle ranging from 0\B0 at the PrimeMeredian to 360\B0, rotating CCW seen from the North pole
	 * @param timeInMillis	timeInMillis (UTC as produced by java.Calendar)
	 * @return String[] with solarTime and a list of planet names in descending distance order
	 *
	 */
	public String[] getSky(String planet,  double latitude, double longitude, long timeInMillis) {
		PlanetDataSource.PlanetDataStructure myPlanet = pd.planetDataTable.get(planet);
		double[] myPlanetXYZ = getXYZ(planet, timeInMillis);

		double[] upwardDirection = getUpwardDirection(planet, latitude,longitude, timeInMillis);

		Rotator rp1 = new Rotator(upwardDirection,myPlanet.rotationAxis,-90);
		double south[] = rp1.rotate(upwardDirection);
		double west[] = rp1.crossproduct(south, upwardDirection);
		String[] result = getSky(myPlanetXYZ, south, west, timeInMillis, myPlanet);
		return result;
	}
	/**  <pre>
	 * Calculate the solar time Find the location of the planets in the sky for an observer on a planet at given latitude and longitude and time
	 * 
	 * @param sun      vector pointing to the sun (in ecliptic coordinates)
	 * @param myview   vector pointing up at the observer's location (in ecliptic coordinates)
	 * @param myPlanet data of the observer's planet 
	 * @return number of solar hours (local observer planet-hours != earth hours) since last midnight
	 */
	public double solarTime(double[] sun, double[] myview, PlanetDataSource.PlanetDataStructure myPlanet) {
		// Equatorial sundials : The distinguishing characteristic of the equatorial dial
		// (also called the equinoctial dial) is the planar surface that receives the shadow,
		// which is exactly perpendicular to the gnomon's style. This plane is called equatorial,
		// because it is parallel to the equator of the Earth and of the celestial sphere.
		// (wikipedia)
		//set up a rotator that rotates the ecliptic z-axis to equarorial/celestial coordinates
		double[] northpole = myPlanet.rotationAxis; 
		Rotator rp = new Rotator(northpole, 0);
		if (northpole[0] != 0 || northpole[1] != 0) {
			double[] r = rp.crossproduct(new double[] {0, 0,1}, northpole);
			double angle = angle(new double[] {0, 0,1}, northpole);
			rp = new Rotator(r, -angle);
		}
		double[] sunCelestial = rp.rotate(sun);
		double[] myviewCelestial = rp.rotate(myview);
		double angle = Math.atan2(myviewCelestial[1],myviewCelestial[0])-Math.atan2(sunCelestial[1],sunCelestial[0]);
		angle = Math.toDegrees(angle)+180.;
		while (angle < 0) angle = angle+360;
		while (angle > 360) angle = angle-360;
		// bereken een tijd uit deze hoek
		// ltime is the number of siderial hours since last midnight,
		//          where a siderial hour is 1/24th of a siderial day 
		double ltime = angle*24/360;

		// a siderial hour is for non-retrogade planets slightly shorter than a solar hour
		double siderialDay = myPlanet.siderialDay; // in solarEarthDays - negative for retrogade planets 
		double solarYear = myPlanet.orbitalPeriod*pd.daysPerYear; // in solarEarthDays
		double siderialDaysPerYear = solarYear/siderialDay;
		double solarDaysPerYear = siderialDaysPerYear-1;

		double solarTime = ltime*solarDaysPerYear/siderialDaysPerYear;
		return solarTime;
	}
	/**  <pre>
	 * Convert solar time as double to conventional string notation
	 * @param solarTime number of solar hours (in planet-hours != earth hours) since last midnight
	 * @return solar time
	 */
	public String solarTimeToString(double solarTime) {
		// bereken een tijd uit deze hoek
		// double ltime = tangle*24/360;
		int hours = (int)solarTime;
		int minutes = (int)((solarTime-hours)*60);

		String solarTimeString = "";
		if (hours < 12) {
			solarTimeString = ""+hours+":"+String.format("%02d", minutes)+" AM";
		} else {
			solarTimeString = ""+(hours-12)+":"+String.format("%02d", minutes)+" PM";
		}
		return solarTimeString;
	}
	/**  <pre>
	 * vector print voor diagnose
	 * @param i
	 * @param p
	 */
	void print(String i, double[] p) {
		for (int j = 0; j < p.length; j++) {
			i = i + "\t"+p[j];
		}
		System.out.println(i);
	}

	double[] normalise(int l, double[] p) {
		double[] n = new double[l];
		double length = 0;
		for (int i = 0; i<l; i++) {
			length = length+p[i]*p[i];
		}
		length = Math.sqrt(length);
		for (int i = 0; i<l; i++) {
			n[i] = p[i]/length;
		}
		return n;
	}
	/**  <pre>
	 * Find upward direction on a planet at given latitude and longitude and time 
	 * @param planet	name of the planet
	 * @param latitude  angle ranging from 0\B0 at the Equator to 90\B0 at the North Pole and -90\B0 at the South Pole
	 * @param longitude	angle ranging from 0\B0 at the PrimeMeredian to 360\B0, rotating CCW seen from the North pole
	 * @param timeInMillis	timeInMillis produced by java.Calendar
	 * @return X,Y,Z coordinates (ecliptic coordinates) of the upward direction at given latitude and longitude and time 
	 */
	public double[] getUpwardDirection(String planet,  double latitude, double longitude, long timeInMillis) {

		PlanetDataSource.PlanetDataStructure myPlanet = pd.planetDataTable.get(planet);

		// get a starting position at the time at which primeMeridianToFirstPointOfARies
		double[] r = myPlanet.rotationAxis;
		double[] aries = {1,0,0};
		double angle = 90 - latitude;
		Rotator x1 = new Rotator(r,aries,angle); 
		double upward[] = x1.rotate(r);

		// determine number of rotation since primeMeridianToFirstPointOfARies
		double siderialDay = myPlanet.siderialDay; // in days
		if (myPlanet.rotationDirection.equals("CW")) {
			siderialDay = -siderialDay;
		}
		long timesinceFPA = timeInMillis-myPlanet.primeMeridianToFirstPointOfARies;
		long siderialDayInMillis = (long)(siderialDay*24.*3600*1000.);

		double numberOfSiderialDaysSinceRApointsToFirstAries = (double)timesinceFPA/(double)siderialDayInMillis;
		double rotationFraction = numberOfSiderialDaysSinceRApointsToFirstAries-(int)numberOfSiderialDaysSinceRApointsToFirstAries;
		// add longitude to the number of rotations
		double rotation = rotationFraction*360.+longitude;
		//create rotator x2 to rotate around the planet axis
		Rotator x2 = new Rotator(r, Math.toRadians(rotation));
		upward = x2.rotate(upward);

		return upward;
	}
	/**  <pre>
	 * Get the ecliptic XYZ coordinates of a planet at a given time
	 * @param planet		name of the planet
	 * @param timeInMillis	timeInMillis produced by java.Calendar
	 * @return
	 */
	public double[] getXYZ(String planet, long timeInMillis) {
		PlanetDataSource.PlanetDataStructure pdd = pd.planetDataTable.get(planet);
		double[] xyz = pdd.po.getXYZcoords(timeInMillis);
		while (! pdd.name.equalsIgnoreCase(pdd.rotationCenter)) {
			pdd = pd.planetDataTable.get(pdd.rotationCenter);
			double[] xyzRC =  pdd.po.getXYZcoords(timeInMillis);
			xyz[0] = xyz[0]+xyzRC[0];
			xyz[1] = xyz[1]+xyzRC[1];
			xyz[2] = xyz[2]+xyzRC[2];
		}
		return xyz;
	} 
	/**  <pre>
	 * Right ascension (abbreviated RA) is the angular distance
	 * of a particular point measured eastward along the celestial equator (= earth equator)
	 * First Point of Aries (Sun at the March equinox)
	 * Declination's angle is measured north or south of the celestial equator
	 *
	 * @param xyzPlanet xyz coordinates of planet for which ra and decl should be determined
	 * @param xyzEarth  xyz coordinates of earth
	 * @return result[3]
	 * result[0] is right ascension in (rad)
	 * result[1] is declination (rad)
	 * result[2] is the distance from earth (km)
	 */
	double[] getRAandDeclination(double[] xyzPlanet, double[] xyzEarth) {
		double[] xyz = new double[3];
		for (int i = 0; i<3; i++) {
			xyz[i] = xyzPlanet[i] - xyzEarth[i];
		}
		xyz = incline.rotate(xyz);
		double[] result = new double[3];
		//right ascension
		result[0] = Math.atan2(xyz[1], xyz[0]);
		// declination
		result[1]  = Math.atan2(xyz[2], Math.sqrt(xyz[0]*xyz[0]+xyz[1]*xyz[1]));
		//result[1]  = Math.atan(xyzPlanet[2]/Math.sqrt(xyzPlanet[0]*xyzPlanet[0]+xyzPlanet[1]*xyzPlanet[1]));
		// distance
		result[2]  = Math.sqrt(xyz[0]*xyz[0]+xyz[1]*xyz[1]+xyz[2]*xyz[2]);
		return result;
	}

	/**  <pre>
	 * converts ra from rad to hours-notation
	 * @param ra right ascension (rad)
	 * @return ra in hours-notation
	 */
	String raToHour(double ra) {
		if (ra < 0) ra = ra + 2.*Math.PI;
		ra = Math.toDegrees(ra);
		ra = ra/15.;
		int hours = (int)ra;
		int minutes = (int)(60*(ra - hours));
		int seconds = (int)(60*(60*(ra - hours)-minutes));
		return hours+"h"+minutes+"m"+seconds+"s";
	}
	/**  <pre>
	 * Calculate solartime on earth (diagnose tool)
	 * @param longitude	angle ranging from 0\B0 at the PrimeMeredian to 360\B0, rotating CCW seen from the North pole
	 * @param timeInMillis
	 * @return solarTime
	 */
	String SolarTimeOnEarth(double longitude, long timeInMillis) {
		Calendar utc = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
		utc.setTimeInMillis(timeInMillis);
		//System.out.println("utc ="+utc.get(Calendar.HOUR_OF_DAY)+":"+String.format("%02d", utc.get(Calendar.MINUTE))+" "+utc.get(Calendar.DAY_OF_MONTH)+"-"+(utc.get(Calendar.MONTH)+1)+"-"+utc.get(Calendar.YEAR));
		timeInMillis=timeInMillis+(long)(longitude*1000.*3600.*24./360.);
		utc.setTimeInMillis(timeInMillis);
		String solarTime = utc.get(Calendar.HOUR_OF_DAY)+":"+String.format("%02d", utc.get(Calendar.MINUTE))+" "+utc.get(Calendar.DAY_OF_MONTH)+"-"+(utc.get(Calendar.MONTH)+1)+"-"+utc.get(Calendar.YEAR);
		//System.out.println("solarTime ="+utc.get(Calendar.HOUR_OF_DAY)+":"+String.format("%02d", utc.get(Calendar.MINUTE))+" "+utc.get(Calendar.DAY_OF_MONTH)+"-"+(utc.get(Calendar.MONTH)+1)+"-"+utc.get(Calendar.YEAR));
		return solarTime;
	}
}