package com.jacquesb.planetario_a;

/**
 *                  ___________________________________________________________
 *  elevation = 90 |                                                           |
 *                 |              __________________________                   |
 *                 | yScreen=0    | (az.Origin,el.Origin)  |                   |
 *                 |              |   x   =(xs0,ys0)       |                   |
 *                 |              |                        |                   |
 *                 |              |                        |                   |
 *                 | yS = hScreen |________________________|                   |
 *                 |         xScreen=0           xScreen=wScreen               |
 *                 |                                                           |
 *                 |                                                           |
 *                 |                                                           |
 *                 |                                                           |
 * elevation = -90 |___________________________________________________________|
 *       azimuth = 0                                                 azimuth = 360
 *
 */
public class ScreenCoords {
    double azimuth, elevation;
    double xScreen, yScreen;
    double wScreen,hScreen;
    double azimuthOrigin,elevationOrigin;
    double xs0,ys0;
    double zoomFactor;
    private double screen2azimuth;
    private double azimuth2screen;
    void setScreen(double wScreen,double hScreen) {
        this.wScreen=wScreen;
        this.hScreen=hScreen;
    }
    void setZoomOrigin(double azimuthOrigin, double elevationOrigin, double xs0, double ys0, double zoomFactor) {
        this.azimuthOrigin=azimuthOrigin;
        this.elevationOrigin=elevationOrigin;
        this.xs0 = xs0;
        this.ys0 = ys0;
        this.zoomFactor=zoomFactor;
        if (wScreen < 2*hScreen) {
            // zoomfactor =1: w -> 360: screen2azimuth = 360/wScreen
            // zoomfactor =2: w -> 180: screen2azimuth = 180/wScreen
            screen2azimuth = 360 / (wScreen * zoomFactor);
            azimuth2screen=1./screen2azimuth;
        } else {
            // zoomfactor =1: h -> 180: screen2azimuth = 180/hScreen
            // zoomfactor =2: h -> 90: screen2azimuth = 90/hScreen
            screen2azimuth = 180 / (hScreen * zoomFactor);
            azimuth2screen=1./screen2azimuth;
        }
        getXscreenYscreen(azimuthOrigin, elevationOrigin);
    }
    void getXscreenYscreen(double azimuth, double elevation) {
    	this.azimuth = azimuth;
    	this.elevation = elevation;
        xScreen = azimuth2screen*(azimuth-azimuthOrigin)+xs0;
        yScreen = ys0-azimuth2screen*(elevation-elevationOrigin);
    }
    void getAzimuthElevation(double xScreen, double yScreen) {
    	this.xScreen = xScreen;
    	this.yScreen = yScreen;
        azimuth= ((xScreen-xs0)/azimuth2screen) + azimuthOrigin;
        elevation= ((ys0-yScreen)/azimuth2screen) + elevationOrigin;
    }
    
	public static void main(String[] args) {
		System.out.println("====================");
		System.out.println("Self-test of Screencoords");
		System.out.println("====================");

        ScreenCoords coords = new ScreenCoords();
        
        System.out.println("coords.setScreen(240, 140)");
		System.out.println("Zoomfactor 1, origen is (180,0)");
        coords.setScreen(240, 140);
        coords.setZoomOrigin(180, 0, 120, 70, 1);
		System.out.println("Zoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");
        
        coords.getAzimuthElevation(120,70);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.getAzimuthElevation(0,70);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");

        coords.getXscreenYscreen(0, 90);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.getAzimuthElevation(125,25);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");

        coords.setZoomOrigin(180, 0, 120, 70, 2);
		System.out.println("\nZoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");

        coords.getAzimuthElevation(125,25);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        coords.getAzimuthElevation(130,-20);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.setZoomOrigin(coords.azimuth, coords.elevation, 120, 70, 2);
		System.out.println("\nZoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.getXscreenYscreen(coords.azimuth-10, coords.elevation-10);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");

        System.out.println("\ncoords.setScreen(360, 180)");
        coords.setScreen(360, 180);
        coords.setZoomOrigin(180, 0, 120, 70, 1);
		System.out.println("Zoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");
        
        coords.getAzimuthElevation(125,25);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        coords.getAzimuthElevation(130,20);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.setZoomOrigin(coords.azimuth, coords.elevation, 120, 70, 2);
		System.out.println("\nZoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        
        coords.getXscreenYscreen(coords.azimuth-10, coords.elevation-10);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");

        System.out.println("\ncoords.setScreen(1920, 996)");
        coords.setScreen(1920, 996);
        coords.setZoomOrigin(180, 0, 120, 70, 1);
		System.out.println("Zoomfactor "+coords.zoomFactor+", origen is ("+coords.azimuth+" degrees, "+coords.elevation+" degrees)");

        System.out.println("QQQ ("+955+","+458+") != ("+coords.xScreen+" ,"+coords.yScreen+") - {"+coords.wScreen+", "+coords.hScreen+"}");
        coords.getAzimuthElevation(955, 458);
        System.out.println(""+coords.azimuth+" degrees, "+coords.elevation+" degrees : ("+coords.xScreen+", "+coords.yScreen+")");
        coords.setZoomOrigin(coords.azimuth, coords.elevation, 120, 70,1);
        System.out.println("QQQ ("+coords.xScreen+","+coords.yScreen+") : az="+coords.azimuth+" el="+coords.elevation+" zoom: "+1);

        
	}
}
