package com.jacquesb.planetario_a;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.TimeZone;

public class SpeedFactor {
    long nextMidNight = -1;
    long prevMidNight = -1;
    int count = 0;
    String speedLabel = "1x";
    long solarDayInMillis;
    Calendar localDateTime;

    ArrayList<Double> previousSpeeds = new ArrayList<Double>();
    SpeedFactor() {
        reset();
        localDateTime = Calendar.getInstance(TimeZone.getDefault());
    }
    void reset() {
        speedLabel = "1x";
        AppData.Data.timeSpeed = 1;
        previousSpeeds = new ArrayList<Double>();
    }
    void stop() {
        reset();
        slowDown();
    }
    void setSolarDayInMillis(long solarDayInMillis) {
        this.solarDayInMillis = solarDayInMillis;
    }
    double subTimeStepInMillis = 0;
    void setNextTime() {
        double timeStepInMillis = AppData.Data.timerStepInMillis*AppData.Data.timeSpeed;
        int solarDaysPerTimeStep = (int)(timeStepInMillis/solarDayInMillis);
        if (Math.abs(solarDaysPerTimeStep) > 0) {
            timeStepInMillis = solarDaysPerTimeStep*solarDayInMillis;
        } else {
            double solarDaysShownPerSecond = (1000.*AppData.Data.timeSpeed/solarDayInMillis);
            if (Math.abs(solarDaysShownPerSecond) > 0.5) {
                subTimeStepInMillis = subTimeStepInMillis + timeStepInMillis;
                if (Math.abs(subTimeStepInMillis) > solarDayInMillis) {
                    subTimeStepInMillis = 0;
                    timeStepInMillis = solarDayInMillis;
                } else {
                    timeStepInMillis = 0;
                }
            }
        }
        AppData.Data.timeInMillis = AppData.Data.timeInMillis+(long)timeStepInMillis;

        localDateTime.setTimeInMillis(AppData.Data.timeInMillis);
        speedLabel= ""+localDateTime.get(Calendar.YEAR);
        // 3 months per second  = 3*3600*24*31 =8034200
        if (Math.abs(AppData.Data.timeSpeed) < 6*8034200) speedLabel= speedLabel+"-"+String.format("%02d", localDateTime.get(Calendar.MONTH)+1);
        // 3 days per second  = 3*3600*24*31 = 259200
        if (Math.abs(AppData.Data.timeSpeed) < 6*259200) speedLabel= speedLabel+"-"+String.format("%02d", localDateTime.get(Calendar.DAY_OF_MONTH));
        // 3 hours per second = 10800
        if (Math.abs(AppData.Data.timeSpeed) < 6*10800) speedLabel= speedLabel+" "+String.format("%02d", localDateTime.get(Calendar.HOUR_OF_DAY))+"h";
        // 3 minutes per second = 180
        if (Math.abs(AppData.Data.timeSpeed) < 6*180) speedLabel= speedLabel+":"+String.format("%02d", localDateTime.get(Calendar.MINUTE))+"m";
        if (Math.abs(AppData.Data.timeSpeed) < 6*3) speedLabel= speedLabel+":"+String.format("%02d", localDateTime.get(Calendar.SECOND))+"s";
    }

    void increaseSpeed() {
        if (AppData.Data.timeSpeed == 0) {
            AppData.Data.timeSpeed = 0.5;
        }
        if (AppData.Data.timeSpeed >= 0) {
            speedUp();
        } else {
            slowDown();
        }
    }
    void decreaseSpeed() {
        if (AppData.Data.timeSpeed == 0) {
            AppData.Data.timeSpeed = -0.5;
        }
            if (AppData.Data.timeSpeed < 0) {
                speedUp();
            } else {
                slowDown();
            }
    }
    void speedUp() {
        if (Math.abs(AppData.Data.timeSpeed) > 0.99) previousSpeeds.add(AppData.Data.timeSpeed);
        if (AppData.Data.timeSpeed == 1) {
            AppData.Data.timeSpeed = 3600;
        } else if (AppData.Data.timeSpeed == -1) {
            AppData.Data.timeSpeed = -3600;
        } else    {
                AppData.Data.timeSpeed = AppData.Data.timeSpeed * 2;
        }
        speedLabel = ""+AppData.Data.timeSpeed+"x";
    }
    void slowDown() {
        int speedStage = previousSpeeds.size();
        if (speedStage == 0) {
            // this means speedfactor = 1x or -1x
            AppData.Data.timeSpeed = 0;
        } else {
            AppData.Data.timeSpeed = previousSpeeds.get(previousSpeeds.size() - 1);
            previousSpeeds.remove(previousSpeeds.size() - 1);
        }
        speedLabel = ""+AppData.Data.timeSpeed+"x";
    }
}
